@@ -1,4 +1,4 @@
-"""Export cli to __main__ for use like python -m sqfluff."""
+"""Export cli to __main__ for use like python -m sqlfluff."""
 from sqlfluff.cli.commands import cli
 
 if __name__ == "__main__":
