@@ -1,7 +1,14 @@
 """Elements which wrap the sqlfluff core library for public use."""
 
-# flake8: noqa: F401
-
 # Expose the simple api
 from sqlfluff.api.simple import lint, fix, parse, APIParsingError
 from sqlfluff.api.info import list_rules, list_dialects
+
+__all__ = (
+    "lint",
+    "fix",
+    "parse",
+    "APIParsingError",
+    "list_rules",
+    "list_dialects",
+)
