@@ -1,12 +1,13 @@
 """Contains the CLI."""
 
 from itertools import chain
+import os
 import sys
 import json
 import logging
 import time
 from logging import LogRecord
-from typing import Callable, Tuple, NoReturn, Optional, List, cast
+from typing import Callable, Tuple, Optional, cast
 
 import yaml
 
@@ -21,17 +22,17 @@ import colorama
 from tqdm import tqdm
 from sqlfluff.cli.autocomplete import shell_completion_enabled, dialect_shell_complete
 
-from sqlfluff.cli.formatters import (
-    format_rules,
-    format_violation,
-    format_linting_result_header,
-    format_linting_stats,
-    colorize,
-    format_dialect_warning,
-    format_dialects,
-    CallbackFormatter,
+from sqlfluff.cli import EXIT_SUCCESS, EXIT_ERROR, EXIT_FAIL
+from sqlfluff.cli.click_deprecated_option import (
+    DeprecatedOption,
+    DeprecatedOptionsCommand,
 )
-from sqlfluff.cli.helpers import cli_table, get_package_version
+from sqlfluff.cli.formatters import (
+    format_linting_result_header,
+    OutputStreamFormatter,
+)
+from sqlfluff.cli.helpers import get_package_version
+from sqlfluff.cli.outputstream import make_output_stream, OutputStream
 
 # Import from sqlfluff core.
 from sqlfluff.core import (
@@ -42,24 +43,11 @@ from sqlfluff.core import (
     SQLFluffUserError,
     dialect_selector,
     dialect_readout,
-    TimingSummary,
 )
 from sqlfluff.core.config import progress_bar_configuration
 
 from sqlfluff.core.enums import FormatType, Color
-from sqlfluff.core.linter import ParsedString
 from sqlfluff.core.plugin.host import get_plugin_manager
-from sqlfluff.core.linter import LintingResult
-
-
-class RedWarningsFilter(logging.Filter):
-    """This filter makes all warnings or above red."""
-
-    def filter(self, record: logging.LogRecord) -> bool:
-        """Filter any warnings (or above) to turn them red."""
-        if record.levelno >= logging.WARNING:
-            record.msg = f"{colorize(record.msg, Color.red)} "
-        return True
 
 
 class StreamHandlerTqdm(logging.StreamHandler):
@@ -81,7 +69,10 @@ class StreamHandlerTqdm(logging.StreamHandler):
 
 
 def set_logging_level(
-    verbosity: int, logger: Optional[logging.Logger] = None, stderr_output: bool = False
+    verbosity: int,
+    formatter: OutputStreamFormatter,
+    logger: Optional[logging.Logger] = None,
+    stderr_output: bool = False,
 ) -> None:
     """Set up logging for the CLI.
 
@@ -108,8 +99,16 @@ def set_logging_level(
     # NB: the unicode character at the beginning is to squash any badly
     # tamed ANSI colour statements, and return us to normality.
     handler.setFormatter(logging.Formatter("\u001b[0m%(levelname)-10s %(message)s"))
+
     # Set up a handler to colour warnings red.
-    handler.addFilter(RedWarningsFilter())
+    # See: https://docs.python.org/3/library/logging.html#filter-objects
+    def red_log_filter(record: logging.LogRecord) -> bool:
+        if record.levelno >= logging.WARNING:
+            record.msg = f"{formatter.colorize(record.msg, Color.red)} "
+        return True
+
+    handler.addFilter(red_log_filter)
+
     if logger:
         focus_logger = logging.getLogger(f"sqlfluff.{logger}")
         focus_logger.addHandler(handler)
@@ -135,6 +134,27 @@ def set_logging_level(
         parser_logger.setLevel(logging.DEBUG)
 
 
+class PathAndUserErrorHandler:
+    """Make an API call but with error handling for the CLI."""
+
+    def __init__(self, formatter):
+        self.formatter = formatter
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        if exc_type is SQLFluffUserError:
+            click.echo(
+                "\nUser Error: "
+                + self.formatter.colorize(
+                    str(exc_val),
+                    Color.red,
+                )
+            )
+            sys.exit(EXIT_ERROR)
+
+
 def common_options(f: Callable) -> Callable:
     """Add common options to commands via a decorator.
 
@@ -173,18 +193,21 @@ def core_options(f: Callable) -> Callable:
     # that supports it
     if shell_completion_enabled:
         f = click.option(
+            "-d",
             "--dialect",
             default=None,
-            help="The dialect of SQL to lint (default=ansi)",
+            help="The dialect of SQL to lint",
             shell_complete=dialect_shell_complete,
         )(f)
     else:  # pragma: no cover
         f = click.option(
+            "-d",
             "--dialect",
             default=None,
-            help="The dialect of SQL to lint (default=ansi)",
+            help="The dialect of SQL to lint",
         )(f)
     f = click.option(
+        "-t",
         "--templater",
         default=None,
         help="The templater to use (default=jinja)",
@@ -198,6 +221,7 @@ def core_options(f: Callable) -> Callable:
         ),
     )(f)
     f = click.option(
+        "-r",
         "--rules",
         default=None,
         help=(
@@ -209,6 +233,7 @@ def core_options(f: Callable) -> Callable:
         ),
     )(f)
     f = click.option(
+        "-e",
         "--exclude-rules",
         default=None,
         help=(
@@ -245,13 +270,14 @@ def core_options(f: Callable) -> Callable:
     )(f)
     f = click.option(
         "--encoding",
-        default="autodetect",
+        default=None,
         help=(
             "Specify encoding to use when reading and writing files. Defaults to "
             "autodetect."
         ),
     )(f)
     f = click.option(
+        "-i",
         "--ignore",
         default=None,
         help=(
@@ -271,7 +297,8 @@ def core_options(f: Callable) -> Callable:
     f = click.option(
         "--logger",
         type=click.Choice(
-            ["templater", "lexer", "parser", "linter", "rules"], case_sensitive=False
+            ["templater", "lexer", "parser", "linter", "rules", "config"],
+            case_sensitive=False,
         ),
         help="Choose to limit the logging to one of the loggers.",
     )(f)
@@ -290,26 +317,33 @@ def get_config(
     **kwargs,
 ) -> FluffConfig:
     """Get a config object from kwargs."""
-    if "dialect" in kwargs:
+    plain_output = OutputStreamFormatter.should_produce_plain_output(kwargs["nocolor"])
+    if kwargs.get("dialect"):
         try:
             # We're just making sure it exists at this stage.
             # It will be fetched properly in the linter.
             dialect_selector(kwargs["dialect"])
         except SQLFluffUserError as err:
             click.echo(
-                colorize(
+                OutputStreamFormatter.colorize_helper(
+                    plain_output,
                     f"Error loading dialect '{kwargs['dialect']}': {str(err)}",
                     color=Color.red,
                 )
             )
-            sys.exit(66)
+            sys.exit(EXIT_ERROR)
         except KeyError:
             click.echo(
-                colorize(
-                    f"Error: Unknown dialect '{kwargs['dialect']}'", color=Color.red
+                OutputStreamFormatter.colorize_helper(
+                    plain_output,
+                    f"Error: Unknown dialect '{kwargs['dialect']}'",
+                    color=Color.red,
                 )
             )
-            sys.exit(66)
+            sys.exit(EXIT_ERROR)
+    from_root_kwargs = {}
+    if "require_dialect" in kwargs:
+        from_root_kwargs["require_dialect"] = kwargs.pop("require_dialect")
     # Instantiate a config object (filtering out the nulls)
     overrides = {k: kwargs[k] for k in kwargs if kwargs[k] is not None}
     try:
@@ -317,74 +351,60 @@ def get_config(
             extra_config_path=extra_config_path,
             ignore_local_config=ignore_local_config,
             overrides=overrides,
+            **from_root_kwargs,
         )
     except SQLFluffUserError as err:  # pragma: no cover
         click.echo(
-            colorize(
+            OutputStreamFormatter.colorize_helper(
+                plain_output,
                 f"Error loading config: {str(err)}",
                 color=Color.red,
             )
         )
-        sys.exit(66)
-
-
-def _callback_handler(cfg: FluffConfig) -> Callable:
-    """Returns function which will be bound as a callback for printing passed message.
-
-    Called in `get_linter_and_formatter`.
-    """
-
-    def _echo_with_tqdm_lock(message: str) -> None:
-        """Makes sure that message printing (echoing) will be not in conflict with tqdm.
-
-        It may happen that progressbar conflicts with extra printing. Nothing very
-        serious happens then, except that there is printed (not removed) progressbar
-        line. The `external_write_mode` allows to disable tqdm for writing time.
-        """
-        with tqdm.external_write_mode():
-            click.echo(message=message, color=cfg.get("color"))
-
-    return _echo_with_tqdm_lock
+        sys.exit(EXIT_ERROR)
 
 
 def get_linter_and_formatter(
-    cfg: FluffConfig, silent: bool = False
-) -> Tuple[Linter, CallbackFormatter]:
+    cfg: FluffConfig, output_stream: Optional[OutputStream] = None
+) -> Tuple[Linter, OutputStreamFormatter]:
     """Get a linter object given a config."""
     try:
         # We're just making sure it exists at this stage.
         # It will be fetched properly in the linter.
-        dialect_selector(cfg.get("dialect"))
+        dialect = cfg.get("dialect")
+        if dialect:
+            dialect_selector(dialect)
     except KeyError:  # pragma: no cover
         click.echo(f"Error: Unknown dialect '{cfg.get('dialect')}'")
-        sys.exit(66)
-
-    if not silent:
-        # Instantiate the linter and return it (with an output function)
-        formatter = CallbackFormatter(
-            callback=_callback_handler(cfg=cfg),
-            verbosity=cfg.get("verbose"),
-            output_line_length=cfg.get("output_line_length"),
-        )
-        return Linter(config=cfg, formatter=formatter), formatter
-    else:
-        # Instantiate the linter and return. NB: No formatter
-        # in the Linter and a black formatter otherwise.
-        formatter = CallbackFormatter(callback=lambda m: None, verbosity=0)
-        return Linter(config=cfg), formatter
+        sys.exit(EXIT_ERROR)
+    formatter = OutputStreamFormatter(
+        output_stream=output_stream or make_output_stream(cfg),
+        nocolor=cfg.get("nocolor"),
+        verbosity=cfg.get("verbose"),
+        output_line_length=cfg.get("output_line_length"),
+    )
+    return Linter(config=cfg, formatter=formatter), formatter
 
 
-@click.group(context_settings={"help_option_names": ["-h", "--help"]})
+@click.group(
+    context_settings={"help_option_names": ["-h", "--help"]},
+    epilog="""\b\bExamples:\n
+  sqlfluff lint --dialect postgres .\n
+  sqlfluff lint --dialect postgres --rules L042 .\n
+  sqlfluff fix --dialect sqlite --rules L041,L042 src/queries\n
+  sqlfluff parse --dialect sqlite --templater jinja src/queries/common.sql
+""",
+)
 @click.version_option()
 def cli():
-    """Sqlfluff is a modular sql linter for humans."""
+    """SQLFluff is a modular SQL linter for humans."""  # noqa D403
 
 
 @cli.command()
 @common_options
 def version(**kwargs) -> None:
     """Show the version of sqlfluff."""
-    c = get_config(**kwargs)
+    c = get_config(**kwargs, require_dialect=False)
     if c.get("verbose") > 0:
         # Instantiate the linter
         lnt, formatter = get_linter_and_formatter(c)
@@ -399,17 +419,18 @@ def version(**kwargs) -> None:
 @common_options
 def rules(**kwargs) -> None:
     """Show the current rules in use."""
-    c = get_config(**kwargs)
-    lnt, _ = get_linter_and_formatter(c)
-    click.echo(format_rules(lnt), color=c.get("color"))
+    c = get_config(**kwargs, dialect="ansi")
+    lnt, formatter = get_linter_and_formatter(c)
+    click.echo(formatter.format_rules(lnt), color=c.get("color"))
 
 
 @cli.command()
 @common_options
 def dialects(**kwargs) -> None:
     """Show the current dialects available."""
-    c = get_config(**kwargs)
-    click.echo(format_dialects(dialect_readout), color=c.get("color"))
+    c = get_config(**kwargs, require_dialect=False)
+    _, formatter = get_linter_and_formatter(c)
+    click.echo(formatter.format_dialects(dialect_readout), color=c.get("color"))
 
 
 def dump_file_payload(filename: Optional[str], payload: str):
@@ -423,7 +444,7 @@ def dump_file_payload(filename: Optional[str], payload: str):
         click.echo(payload)
 
 
-@cli.command()
+@cli.command(cls=DeprecatedOptionsCommand)
 @common_options
 @core_options
 @click.option(
@@ -445,10 +466,10 @@ def dump_file_payload(filename: Optional[str], payload: str):
 @click.option(
     "--annotation-level",
     default="notice",
-    type=click.Choice(["notice", "warning", "failure"], case_sensitive=False),
+    type=click.Choice(["notice", "warning", "failure", "error"], case_sensitive=False),
     help=(
-        "When format is set to github-annotation, "
-        "default annotation level (default=notice)."
+        "When format is set to github-annotation or github-annotation-native, "
+        "default annotation level (default=notice). failure and error are equivalent."
     ),
 )
 @click.option(
@@ -468,18 +489,34 @@ def dump_file_payload(filename: Optional[str], payload: str):
     "-p",
     "--processes",
     type=int,
-    default=1,
-    help="The number of parallel processes to run.",
+    default=None,
+    help=(
+        "The number of parallel processes to run. Positive numbers work as "
+        "expected. Zero and negative numbers will work as number_of_cpus - "
+        "number. e.g  -1 means all cpus except one. 0 means all cpus."
+    ),
 )
 @click.option(
     "--disable_progress_bar",
+    "--disable-progress-bar",
     is_flag=True,
     help="Disables progress bars.",
+    cls=DeprecatedOption,
+    deprecated=["--disable_progress_bar"],
+)
+@click.option(
+    "--persist-timing",
+    default=None,
+    help=(
+        "A filename to persist the timing information for a linting run to "
+        "in csv format for external analysis. NOTE: This feature should be "
+        "treated as beta, and the format of the csv file may change in "
+        "future releases without warning."
+    ),
 )
 @click.argument("paths", nargs=-1, type=click.Path(allow_dash=True))
 def lint(
     paths: Tuple[str],
-    processes: int,
     format: str,
     write_output: Optional[str],
     annotation_level: str,
@@ -487,11 +524,13 @@ def lint(
     disregard_sqlfluffignores: bool,
     logger: Optional[logging.Logger] = None,
     bench: bool = False,
+    processes: Optional[int] = None,
     disable_progress_bar: Optional[bool] = False,
     extra_config_path: Optional[str] = None,
     ignore_local_config: bool = False,
+    persist_timing: Optional[str] = None,
     **kwargs,
-) -> NoReturn:
+) -> None:
     """Lint SQL files via passing a list of files or using stdin.
 
     PATH is the path to a sql file or directory to lint. This can be either a
@@ -510,10 +549,13 @@ def lint(
         echo 'select col from tbl' | sqlfluff lint -
 
     """
-    config = get_config(extra_config_path, ignore_local_config, **kwargs)
+    config = get_config(
+        extra_config_path, ignore_local_config, require_dialect=False, **kwargs
+    )
     non_human_output = (format != FormatType.human.value) or (write_output is not None)
     file_output = None
-    lnt, formatter = get_linter_and_formatter(config, silent=non_human_output)
+    output_stream = make_output_stream(config, format, write_output)
+    lnt, formatter = get_linter_and_formatter(config, output_stream)
 
     verbose = config.get("verbose")
     progress_bar_configuration.disable_progress_bar = disable_progress_bar
@@ -521,39 +563,41 @@ def lint(
     formatter.dispatch_config(lnt)
 
     # Set up logging.
-    set_logging_level(verbosity=verbose, logger=logger, stderr_output=non_human_output)
-    # add stdin if specified via lone '-'
-    if ("-",) == paths:
-        result = lnt.lint_string_wrapped(sys.stdin.read(), fname="stdin")
-    else:
-        # Output the results as we go
-        if verbose >= 1:
-            click.echo(format_linting_result_header())
-        try:
+    set_logging_level(
+        verbosity=verbose,
+        formatter=formatter,
+        logger=logger,
+        stderr_output=non_human_output,
+    )
+
+    # Output the results as we go
+    if verbose >= 1 and not non_human_output:
+        click.echo(format_linting_result_header())
+
+    with PathAndUserErrorHandler(formatter):
+        # add stdin if specified via lone '-'
+        if ("-",) == paths:
+            result = lnt.lint_string_wrapped(sys.stdin.read(), fname="stdin")
+        else:
             result = lnt.lint_paths(
                 paths,
                 ignore_non_existent_files=False,
                 ignore_files=not disregard_sqlfluffignores,
                 processes=processes,
             )
-        except OSError:
-            click.echo(
-                colorize(
-                    f"The path(s) '{paths}' could not be accessed. Check it/they "
-                    "exist(s).",
-                    Color.red,
-                )
-            )
-            sys.exit(1)
-        # Output the final stats
-        if verbose >= 1:
-            click.echo(format_linting_stats(result, verbose=verbose))
+
+    # Output the final stats
+    if verbose >= 1 and not non_human_output:
+        click.echo(formatter.format_linting_stats(result, verbose=verbose))
 
     if format == FormatType.json.value:
         file_output = json.dumps(result.as_records())
     elif format == FormatType.yaml.value:
         file_output = yaml.dump(result.as_records(), sort_keys=False)
     elif format == FormatType.github_annotation.value:
+        if annotation_level == "error":
+            annotation_level = "failure"
+
         github_result = []
         for record in result.as_records():
             filepath = record["filepath"]
@@ -574,48 +618,49 @@ def lint(
                     }
                 )
         file_output = json.dumps(github_result)
+    elif format == FormatType.github_annotation_native.value:
+        if annotation_level == "failure":
+            annotation_level = "error"
+
+        github_result_native = []
+        for record in result.as_records():
+            filepath = record["filepath"]
+            for violation in record["violations"]:
+                # NOTE: The output format is designed for GitHub action:
+                # https://docs.github.com/en/actions/using-workflows/workflow-commands-for-github-actions#setting-a-notice-message
+                line = f"::{annotation_level} "
+                line += "title=SQLFluff,"
+                line += f"file={filepath},"
+                line += f"line={violation['line_no']},"
+                line += f"col={violation['line_pos']}"
+                line += "::"
+                line += f"{violation['code']}: {violation['description']}"
+
+                github_result_native.append(line)
+
+        file_output = "\n".join(github_result_native)
 
     if file_output:
         dump_file_payload(write_output, cast(str, file_output))
 
+    if persist_timing:
+        result.persist_timing_records(persist_timing)
+
+    output_stream.close()
     if bench:
         click.echo("==== overall timings ====")
-        click.echo(cli_table([("Clock time", result.total_time)]))
+        click.echo(formatter.cli_table([("Clock time", result.total_time)]))
         timing_summary = result.timing_summary()
         for step in timing_summary:
             click.echo(f"=== {step} ===")
-            click.echo(cli_table(timing_summary[step].items()))
+            click.echo(formatter.cli_table(timing_summary[step].items()))
 
     if not nofail:
         if not non_human_output:
-            _completion_message(config)
+            formatter.completion_message()
         sys.exit(result.stats()["exit code"])
     else:
-        sys.exit(0)
-
-
-def _handle_files_with_tmp_or_prs_errors(lint_result: LintingResult) -> int:
-    """Discard lint fixes for files with templating or parse errors.
-
-    Returns 1 if there are any files with templating or parse errors after
-    filtering, else 0. (Intended as a process exit code.)
-    """
-    total_errors, num_filtered_errors = lint_result.count_tmp_prs_errors()
-    lint_result.discard_fixes_for_lint_errors_in_files_with_tmp_or_prs_errors()
-    if total_errors:
-        click.echo(
-            colorize(f"  [{total_errors} templating/parsing errors found]", Color.red)
-        )
-        if num_filtered_errors < total_errors:
-            color = Color.red if num_filtered_errors else Color.green
-            click.echo(
-                colorize(
-                    f"  [{num_filtered_errors} templating/parsing errors "
-                    f'remaining after "ignore"]',
-                    color,
-                )
-            )
-    return 1 if num_filtered_errors else 0
+        sys.exit(EXIT_SUCCESS)
 
 
 def do_fixes(lnt, result, formatter=None, **kwargs):
@@ -635,7 +680,7 @@ def do_fixes(lnt, result, formatter=None, **kwargs):
     return False  # pragma: no cover
 
 
-@cli.command()
+@cli.command(cls=DeprecatedOptionsCommand)
 @common_options
 @core_options
 @click.option(
@@ -648,23 +693,34 @@ def do_fixes(lnt, result, formatter=None, **kwargs):
     ),
 )
 @click.option(
-    "--fixed-suffix", default=None, help="An optional suffix to add to fixed files."
+    "-x",
+    "--fixed-suffix",
+    default=None,
+    help="An optional suffix to add to fixed files.",
 )
 @click.option(
     "-p",
     "--processes",
     type=int,
-    default=1,
-    help="The number of parallel processes to run.",
+    default=None,
+    help=(
+        "The number of parallel processes to run. Positive numbers work as "
+        "expected. Zero and negative numbers will work as number_of_cpus - "
+        "number. e.g  -1 means all cpus except one. 0 means all cpus."
+    ),
 )
 @click.option(
     "--disable_progress_bar",
+    "--disable-progress-bar",
     is_flag=True,
     help="Disables progress bars.",
+    cls=DeprecatedOption,
+    deprecated=["--disable_progress_bar"],
 )
 @click.option(
     "--FIX-EVEN-UNPARSABLE",
     is_flag=True,
+    default=None,
     help=(
         "Enables fixing of files that have templating or parse errors. "
         "Note that the similar-sounding '--ignore' or 'noqa' features merely "
@@ -674,19 +730,25 @@ def do_fixes(lnt, result, formatter=None, **kwargs):
         "or in the .sqlfluff config file."
     ),
 )
+@click.option(
+    "--show-lint-violations",
+    is_flag=True,
+    help="Show lint violations",
+)
 @click.argument("paths", nargs=-1, type=click.Path(allow_dash=True))
 def fix(
     force: bool,
     paths: Tuple[str],
-    processes: int,
     bench: bool = False,
     fixed_suffix: str = "",
     logger: Optional[logging.Logger] = None,
+    processes: Optional[int] = None,
     disable_progress_bar: Optional[bool] = False,
     extra_config_path: Optional[str] = None,
     ignore_local_config: bool = False,
+    show_lint_violations: bool = False,
     **kwargs,
-) -> NoReturn:
+) -> None:
     """Fix SQL files.
 
     PATH is the path to a sql file or directory to lint. This can be either a
@@ -697,19 +759,29 @@ def fix(
     # some quick checks
     fixing_stdin = ("-",) == paths
 
-    config = get_config(extra_config_path, ignore_local_config, **kwargs)
+    config = get_config(
+        extra_config_path, ignore_local_config, require_dialect=False, **kwargs
+    )
     fix_even_unparsable = config.get("fix_even_unparsable")
-    lnt, formatter = get_linter_and_formatter(config, silent=fixing_stdin)
+    output_stream = make_output_stream(
+        config, None, os.devnull if fixing_stdin else None
+    )
+    lnt, formatter = get_linter_and_formatter(config, output_stream)
 
     verbose = config.get("verbose")
     progress_bar_configuration.disable_progress_bar = disable_progress_bar
 
-    exit_code = 0
+    exit_code = EXIT_SUCCESS
 
     formatter.dispatch_config(lnt)
 
     # Set up logging.
-    set_logging_level(verbosity=verbose, logger=logger, stderr_output=fixing_stdin)
+    set_logging_level(
+        verbosity=verbose,
+        formatter=formatter,
+        logger=logger,
+        stderr_output=fixing_stdin,
+    )
 
     # handle stdin case. should output formatted sql to stdout and nothing else.
     if fixing_stdin:
@@ -719,7 +791,7 @@ def fix(
         templater_error = result.num_violations(types=SQLTemplaterError) > 0
         unfixable_error = result.num_violations(types=SQLLintError, fixable=False) > 0
         if not fix_even_unparsable:
-            exit_code = _handle_files_with_tmp_or_prs_errors(result)
+            exit_code = formatter.handle_files_with_tmp_or_prs_errors(result)
 
         if result.num_violations(types=SQLLintError, fixable=True) > 0:
             stdout = result.paths[0].files[0].fix_string()[0]
@@ -728,47 +800,42 @@ def fix(
 
         if templater_error:
             click.echo(
-                colorize(
-                    "Fix aborted due to unparseable template variables.",
+                formatter.colorize(
+                    "Fix aborted due to unparsable template variables.",
                     Color.red,
                 ),
                 err=True,
             )
             click.echo(
-                colorize(
-                    "Use --fix-even-unparsable' to attempt to fix the SQL anyway.",
+                formatter.colorize(
+                    "Use --FIX-EVEN-UNPARSABLE' to attempt to fix the SQL anyway.",
                     Color.red,
                 ),
                 err=True,
             )
 
         if unfixable_error:
-            click.echo(colorize("Unfixable violations detected.", Color.red), err=True)
+            click.echo(
+                formatter.colorize("Unfixable violations detected.", Color.red),
+                err=True,
+            )
 
         click.echo(stdout, nl=False)
-        sys.exit(1 if templater_error or unfixable_error else exit_code)
+        sys.exit(EXIT_FAIL if templater_error or unfixable_error else exit_code)
 
     # Lint the paths (not with the fix argument at this stage), outputting as we go.
     click.echo("==== finding fixable violations ====")
-    try:
+
+    with PathAndUserErrorHandler(formatter):
         result = lnt.lint_paths(
             paths,
             fix=True,
             ignore_non_existent_files=False,
             processes=processes,
         )
-    except OSError:
-        click.echo(
-            colorize(
-                f"The path(s) '{paths}' could not be accessed. Check it/they exist(s).",
-                Color.red,
-            ),
-            err=True,
-        )
-        sys.exit(1)
 
     if not fix_even_unparsable:
-        exit_code = _handle_files_with_tmp_or_prs_errors(result)
+        exit_code = formatter.handle_files_with_tmp_or_prs_errors(result)
 
     # NB: We filter to linting violations here, because they're
     # the only ones which can be potentially fixed.
@@ -779,7 +846,9 @@ def fix(
             "linting violations found"
         )
         if force:
-            click.echo(f"{colorize('FORCE MODE', Color.red)}: Attempting fixes...")
+            click.echo(
+                f"{formatter.colorize('FORCE MODE', Color.red)}: Attempting fixes..."
+            )
             success = do_fixes(
                 lnt,
                 result,
@@ -788,7 +857,7 @@ def fix(
                 fixed_file_suffix=fixed_suffix,
             )
             if not success:
-                sys.exit(1)  # pragma: no cover
+                sys.exit(EXIT_FAIL)  # pragma: no cover
         else:
             click.echo(
                 "Are you sure you wish to attempt to fix these? [Y/n] ", nl=False
@@ -805,25 +874,25 @@ def fix(
                     fixed_file_suffix=fixed_suffix,
                 )
                 if not success:
-                    sys.exit(1)  # pragma: no cover
+                    sys.exit(EXIT_FAIL)  # pragma: no cover
                 else:
-                    _completion_message(config)
+                    formatter.completion_message()
             elif c == "n":
                 click.echo("Aborting...")
-                exit_code = 1
+                exit_code = EXIT_FAIL
             else:  # pragma: no cover
                 click.echo("Invalid input, please enter 'Y' or 'N'")
                 click.echo("Aborting...")
-                exit_code = 1
+                exit_code = EXIT_FAIL
     else:
         click.echo("==== no fixable linting violations found ====")
-        _completion_message(config)
+        formatter.completion_message()
 
     error_types = [
         (
             dict(types=SQLLintError, fixable=False),
             "  [{} unfixable linting violations found]",
-            1,
+            EXIT_FAIL,
         ),
     ]
     for num_violations_kwargs, message_format, error_level in error_types:
@@ -834,22 +903,25 @@ def fix(
 
     if bench:
         click.echo("==== overall timings ====")
-        click.echo(cli_table([("Clock time", result.total_time)]))
+        click.echo(formatter.cli_table([("Clock time", result.total_time)]))
         timing_summary = result.timing_summary()
         for step in timing_summary:
             click.echo(f"=== {step} ===")
-            click.echo(cli_table(timing_summary[step].items()))
+            click.echo(formatter.cli_table(timing_summary[step].items()))
+
+    if show_lint_violations:
+        click.echo("==== lint for unfixable violations ====")
+        all_results = result.violation_dict(**num_violations_kwargs)
+        sorted_files = sorted(all_results.keys())
+        for file in sorted_files:
+            violations = all_results.get(file, [])
+            click.echo(formatter.format_filename(file, success=(not violations)))
+            for violation in violations:
+                click.echo(formatter.format_violation(violation))
 
     sys.exit(exit_code)
 
 
-def _completion_message(config: FluffConfig) -> None:
-    click.echo(
-        "All Finished"
-        f"{'' if (config.get('nocolor') or not sys.stdout.isatty()) else ' 📜 🎉'}!"
-    )
-
-
 def quoted_presenter(dumper, data):
     """Re-presenter which always double quotes string values needing escapes."""
     if "\n" in data or "\t" in data or "'" in data:
@@ -926,7 +998,7 @@ def parse(
     extra_config_path: Optional[str] = None,
     ignore_local_config: bool = False,
     **kwargs,
-) -> NoReturn:
+) -> None:
     """Parse SQL files and just spit out the result.
 
     PATH is the path to a sql file or directory to lint. This can be either a
@@ -934,11 +1006,14 @@ def parse(
     character to indicate reading from *stdin* or a dot/blank ('.'/' ') which will
     be interpreted like passing the current working directory as a path argument.
     """
-    c = get_config(extra_config_path, ignore_local_config, **kwargs)
+    c = get_config(
+        extra_config_path, ignore_local_config, require_dialect=False, **kwargs
+    )
     # We don't want anything else to be logged if we want json or yaml output
     # unless we're writing to a file.
     non_human_output = (format != FormatType.human.value) or (write_output is not None)
-    lnt, formatter = get_linter_and_formatter(c, silent=non_human_output)
+    output_stream = make_output_stream(c, format, write_output)
+    lnt, formatter = get_linter_and_formatter(c, output_stream)
     verbose = c.get("verbose")
     recurse = c.get("recurse")
 
@@ -947,7 +1022,12 @@ def parse(
     formatter.dispatch_config(lnt)
 
     # Set up logging.
-    set_logging_level(verbosity=verbose, logger=logger, stderr_output=non_human_output)
+    set_logging_level(
+        verbosity=verbose,
+        formatter=formatter,
+        logger=logger,
+        stderr_output=non_human_output,
+    )
 
     # TODO: do this better
 
@@ -957,14 +1037,14 @@ def parse(
             import cProfile
         except ImportError:  # pragma: no cover
             click.echo("The cProfiler is not available on your platform.")
-            sys.exit(1)
+            sys.exit(EXIT_ERROR)
         pr = cProfile.Profile()
         pr.enable()
 
-    try:
-        t0 = time.monotonic()
+    t0 = time.monotonic()
 
-        # handle stdin if specified via lone '-'
+    # handle stdin if specified via lone '-'
+    with PathAndUserErrorHandler(formatter):
         if "-" == path:
             parsed_strings = [
                 lnt.parse_string(
@@ -976,50 +1056,44 @@ def parse(
             ]
         else:
             # A single path must be specified for this command
-            parsed_strings = list(lnt.parse_path(path, recurse=recurse))
-
-        total_time = time.monotonic() - t0
-        violations_count = 0
-
-        # iterative print for human readout
-        if format == FormatType.human.value:
-            violations_count = _print_out_violations_and_timing(
-                bench, code_only, total_time, verbose, parsed_strings
-            )
-        else:
-            parsed_strings_dict = [
-                dict(
-                    filepath=linted_result.fname,
-                    segments=linted_result.tree.as_record(
-                        code_only=code_only, show_raw=True, include_meta=include_meta
-                    )
-                    if linted_result.tree
-                    else None,
+            parsed_strings = list(
+                lnt.parse_path(
+                    path=path,
+                    recurse=recurse,
                 )
-                for linted_result in parsed_strings
-            ]
+            )
 
-            if format == FormatType.yaml.value:
-                # For yaml dumping always dump double quoted strings if they contain
-                # tabs or newlines.
-                yaml.add_representer(str, quoted_presenter)
-                file_output = yaml.dump(parsed_strings_dict, sort_keys=False)
-            elif format == FormatType.json.value:
-                file_output = json.dumps(parsed_strings_dict)
+    total_time = time.monotonic() - t0
+    violations_count = 0
 
-            # Dump the output to stdout or to file as appropriate.
-            dump_file_payload(write_output, file_output)
-
-    except OSError:  # pragma: no cover
-        click.echo(
-            colorize(
-                f"The path '{path}' could not be accessed. Check it exists.",
-                Color.red,
-            ),
-            err=True,
+    # iterative print for human readout
+    if format == FormatType.human.value:
+        violations_count = formatter.print_out_violations_and_timing(
+            output_stream, bench, code_only, total_time, verbose, parsed_strings
         )
-        sys.exit(1)
+    else:
+        parsed_strings_dict = [
+            dict(
+                filepath=linted_result.fname,
+                segments=linted_result.tree.as_record(
+                    code_only=code_only, show_raw=True, include_meta=include_meta
+                )
+                if linted_result.tree
+                else None,
+            )
+            for linted_result in parsed_strings
+        ]
 
+        if format == FormatType.yaml.value:
+            # For yaml dumping always dump double quoted strings if they contain
+            # tabs or newlines.
+            yaml.add_representer(str, quoted_presenter)
+            file_output = yaml.dump(parsed_strings_dict, sort_keys=False)
+        elif format == FormatType.json.value:
+            file_output = json.dumps(parsed_strings_dict)
+
+        # Dump the output to stdout or to file as appropriate.
+        dump_file_payload(write_output, file_output)
     if profiler:
         pr.disable()
         profiler_buffer = StringIO()
@@ -1030,52 +1104,71 @@ def parse(
         click.echo("\n".join(profiler_buffer.getvalue().split("\n")[:50]))
 
     if violations_count > 0 and not nofail:
-        sys.exit(66)  # pragma: no cover
+        sys.exit(EXIT_FAIL)  # pragma: no cover
     else:
-        sys.exit(0)
+        sys.exit(EXIT_SUCCESS)
 
 
-def _print_out_violations_and_timing(
+@cli.command()
+@common_options
+@core_options
+@click.argument("path", nargs=1, type=click.Path(allow_dash=True))
+def render(
+    path: str,
     bench: bool,
-    code_only: bool,
-    total_time: float,
-    verbose: int,
-    parsed_strings: List[ParsedString],
-) -> int:
-    """Used by human formatting during the parse."""
-    violations_count = 0
-    timing = TimingSummary()
+    logger: Optional[logging.Logger] = None,
+    extra_config_path: Optional[str] = None,
+    ignore_local_config: bool = False,
+    **kwargs,
+) -> None:
+    """Render SQL files and just spit out the result.
 
-    for parsed_string in parsed_strings:
-        timing.add(parsed_string.time_dict)
+    PATH is the path to a sql file. This should be either a single file
+    file ('path/to/file.sql') or a single ('-') character to indicate reading
+    from *stdin*.
+    """
+    c = get_config(
+        extra_config_path, ignore_local_config, require_dialect=False, **kwargs
+    )
+    # We don't want anything else to be logged if we want json or yaml output
+    # unless we're writing to a file.
+    output_stream = make_output_stream(c, None, None)
+    lnt, formatter = get_linter_and_formatter(c, output_stream)
+    verbose = c.get("verbose")
 
-        if parsed_string.tree:
-            click.echo(parsed_string.tree.stringify(code_only=code_only))
+    progress_bar_configuration.disable_progress_bar = True
+
+    formatter.dispatch_config(lnt)
+
+    # Set up logging.
+    set_logging_level(
+        verbosity=verbose,
+        formatter=formatter,
+        logger=logger,
+        stderr_output=False,
+    )
+
+    # handle stdin if specified via lone '-'
+    with PathAndUserErrorHandler(formatter):
+        if "-" == path:
+            raw_sql = sys.stdin.read()
+            fname = "stdin"
+            file_config = lnt.config
         else:
-            # TODO: Make this prettier
-            click.echo("...Failed to Parse...")  # pragma: no cover
+            raw_sql, file_config, _ = lnt.load_raw_file_and_config(path, lnt.config)
+            fname = path
 
-        violations_count += len(parsed_string.violations)
-        if parsed_string.violations:
-            click.echo("==== parsing violations ====")  # pragma: no cover
-        for v in parsed_string.violations:
-            click.echo(format_violation(v))  # pragma: no cover
-        if parsed_string.violations and parsed_string.config.get("dialect") == "ansi":
-            click.echo(format_dialect_warning())  # pragma: no cover
+    # Get file specific config
+    file_config.process_raw_file_for_config(raw_sql)
+    rendered = lnt.render_string(raw_sql, fname, file_config, "utf8")
 
-        if verbose >= 2:
-            click.echo("==== timings ====")
-            click.echo(cli_table(parsed_string.time_dict.items()))
-
-    if verbose >= 2 or bench:
-        click.echo("==== overall timings ====")
-        click.echo(cli_table([("Clock time", total_time)]))
-        timing_summary = timing.summary()
-        for step in timing_summary:
-            click.echo(f"=== {step} ===")
-            click.echo(cli_table(timing_summary[step].items()))
-
-    return violations_count
+    if rendered.templater_violations:
+        for v in rendered.templater_violations:
+            click.echo(formatter.format_violation(v))
+        sys.exit(EXIT_FAIL)
+    else:
+        click.echo(rendered.templated_file.templated_str)
+        sys.exit(EXIT_SUCCESS)
 
 
 # This "__main__" handler allows invoking SQLFluff using "python -m", which
