@@ -1,6 +1,6 @@
 """Defines the base dialect class."""
 
-from typing import Union, Type
+from typing import Set, Union, Type
 
 from sqlfluff.core.parser import (
     KeywordSegment,
@@ -9,12 +9,11 @@ from sqlfluff.core.parser import (
     StringParser,
 )
 from sqlfluff.core.parser.grammar.base import BaseGrammar
+from sqlfluff.core.parser.matchable import Matchable
 
-DialectElementType = Union[
-    Type[BaseSegment], BaseGrammar, StringParser, SegmentGenerator
-]
+DialectElementType = Union[Type[BaseSegment], Matchable, SegmentGenerator]
 # NOTE: Post expansion, no generators remain
-ExpandedDialectElementType = Union[Type[BaseSegment], StringParser, BaseGrammar]
+ExpandedDialectElementType = Union[Type[BaseSegment], Matchable]
 
 
 class Dialect:
@@ -44,7 +43,7 @@ class Dialect:
         self.inherits_from = inherits_from
         self.root_segment_name = root_segment_name
 
-    def __repr__(self):
+    def __repr__(self):  # pragma: no cover
         return f"<Dialect: {self.name}>"
 
     def expand(self) -> "Dialect":
@@ -63,7 +62,7 @@ class Dialect:
                 with expanded references.
         """
         # Are we already expanded?
-        if self.expanded:
+        if self.expanded:  # pragma: no cover
             raise ValueError("Attempted to re-expand an already expanded dialect.")
 
         expanded_copy = self.copy_as(name=self.name)
@@ -89,7 +88,7 @@ class Dialect:
         expanded_copy.expanded = True
         return expanded_copy
 
-    def sets(self, label):
+    def sets(self, label) -> Set:
         """Allows access to sets belonging to this dialect.
 
         These sets belong to the dialect and are copied for sub
@@ -108,7 +107,7 @@ class Dialect:
         `replace` method can be used to override particular rules.
         """
         # Are we already expanded?
-        if self.expanded:
+        if self.expanded:  # pragma: no cover
             # If we copy an already expanded dialect then any SegmentGenerators
             # won't respond. This is most likely a mistake.
             raise ValueError("Attempted to copy an already expanded dialect.")
@@ -127,32 +126,6 @@ class Dialect:
             root_segment_name=self.root_segment_name,
         )
 
-    def segment(self, replace=False):
-        """This is the decorator for elements, it should be called as a method.
-
-        e.g.
-        @dialect.segment()
-        class SomeSegment(BaseSegment):
-            blah blah blah
-
-        """
-
-        def segment_wrap(cls):
-            """Wrap a segment and register it against the dialect."""
-            n = cls.__name__
-            if replace:
-                if n not in self._library:
-                    raise ValueError(f"{n!r} is not already registered in {self!r}")
-            else:
-                if n in self._library:
-                    raise ValueError(f"{n!r} is already registered in {self!r}")
-            self._library[n] = cls
-            # Pass it back after registering it
-            return cls
-
-        # return the wrapping function
-        return segment_wrap
-
     def add(self, **kwargs: DialectElementType):
         """Add a segment to the dialect directly.
 
@@ -166,7 +139,7 @@ class Dialect:
         will iterate through the kwargs
         """
         for n in kwargs:
-            if n in self._library:
+            if n in self._library:  # pragma: no cover
                 raise ValueError(f"{n!r} is already registered in {self!r}")
             self._library[n] = kwargs[n]
 
@@ -176,9 +149,75 @@ class Dialect:
         Usage is very similar to add, but elements specified must already exist.
         """
         for n in kwargs:
-            if n not in self._library:
+            if n not in self._library:  # pragma: no cover
                 raise ValueError(f"{n!r} is not already registered in {self!r}")
-            self._library[n] = kwargs[n]
+            cls = kwargs[n]
+            if self._library[n] is cls:
+                continue
+            elif self._library[n] == cls:
+                # Check for replacement with a new but identical class.
+                # This would be a sign of redundant definitions in the dialect.
+                raise ValueError(
+                    f"Attempted unnecessary identical redefinition of {n!r} in {self!r}"
+                )  # pragma: no cover
+
+            # To replace a segment, the replacement must either be a
+            # subclass of the original, *or* it must have the same
+            # public methods and/or fields as it.
+            base_dir = set(dir(self._library[n]))
+            subclass = False
+            if isinstance(self._library[n], type) and isinstance(cls, type):
+                subclass = issubclass(cls, self._library[n])
+                if not subclass:
+                    if self._library[n].type != cls.type:
+                        raise ValueError(  # pragma: no cover
+                            f"Cannot replace {n!r} because 'type' property does not "
+                            f"match: {cls.type} != {self._library[n].type}"
+                        )
+
+                    cls_dir = set(dir(cls))
+                    missing = set(
+                        n for n in base_dir.difference(cls_dir) if not n.startswith("_")
+                    )
+                    if missing:
+                        raise ValueError(  # pragma: no cover
+                            f"Cannot replace {n!r} because it's not a subclass and "
+                            f"is missing these from base: {', '.join(missing)}"
+                        )
+
+            if subclass:
+                # If the segment class we're replacing defines these fields, the
+                # replacement must override either:
+                # - NONE of them or
+                # - ALL of them
+                # Overriding a subset of them is not necessarily wrong, but it's
+                # error-prone, hence this policy.
+                grammars = {"match_grammar", "parse_grammar"}
+                # TRICKY: The explicit use of __dict__ on the classes is
+                # deliberate. We are concerned with whether a class itself does
+                # or does not define a thing, IGNORING INHERITED VALUES.
+                if grammars.intersection(set(self._library[n].__dict__)) == grammars:
+                    overrides = grammars.intersection(set(cls.__dict__))
+                    if overrides and overrides != grammars:
+                        for grammar in grammars:
+                            if (
+                                grammar in self._library[n].__dict__
+                                and grammar not in cls.__dict__
+                            ):
+                                raise ValueError(
+                                    f"Cannot replace {n!r} because it needs "
+                                    f"to define '{grammar}'"
+                                )
+            self._library[n] = cls
+
+    def add_update_segments(self, module_dct):
+        """Scans module dictionary, adding or replacing segment definitions."""
+        for k, v in module_dct.items():
+            if isinstance(v, type) and issubclass(v, BaseSegment):
+                if k not in self._library:
+                    self.add(**{k: v})
+                else:
+                    self.replace(**{k: v})
 
     def get_grammar(self, name: str) -> BaseGrammar:
         """Allow access to grammars pre-expansion.
@@ -186,9 +225,9 @@ class Dialect:
         This is typically for dialect inheritance. This method
         also validates that the result is a grammar.
         """
-        if name not in self._library:
+        if name not in self._library:  # pragma: no cover
             raise ValueError(f"Element {name} not found in dialect.")
-        if not isinstance(self._library[name], BaseGrammar):
+        if not isinstance(self._library[name], BaseGrammar):  # pragma: no cover
             raise TypeError(
                 f"Attempted to fetch non grammar [{name}] with get_grammar."
             )
@@ -200,9 +239,9 @@ class Dialect:
         This is typically for dialect inheritance. This method
         also validates that the result is a segment.
         """
-        if name not in self._library:
+        if name not in self._library:  # pragma: no cover
             raise ValueError(f"Element {name} not found in dialect.")
-        if not issubclass(self._library[name], BaseSegment):
+        if not issubclass(self._library[name], BaseSegment):  # pragma: no cover
             raise TypeError(
                 f"Attempted to fetch non segment [{name}] with get_segment."
             )
@@ -215,23 +254,33 @@ class Dialect:
         as a result.
 
         """
-        if not self.expanded:
+        if not self.expanded:  # pragma: no cover
             raise RuntimeError("Dialect must be expanded before use.")
 
         if name in self._library:
             res = self._library[name]
             if res:
                 return res
-            else:
+            else:  # pragma: no cover
                 raise ValueError(
                     "Unexpected Null response while fetching {!r} from {}".format(
                         name, self.name
                     )
                 )
-        else:
+        else:  # pragma: no cover
+            if name.endswith("KeywordSegment"):
+                keyword_tip = (
+                    " Perhaps specify the keyword? "
+                    "https://github.com/sqlfluff/sqlfluff/wiki/Contributing-Dialect-Changes#keywords"  # noqa E501
+                )
+            else:
+                keyword_tip = ""
             raise RuntimeError(
-                "Grammar refers to {!r} which was not found in the {} dialect".format(
-                    name, self.name
+                (
+                    "Grammar refers to "
+                    "{!r} which was not found in the {} dialect.{}".format(
+                        name, self.name, keyword_tip
+                    )
                 )
             )
 
@@ -249,7 +298,7 @@ class Dialect:
         """Fetch the lexer struct for this dialect."""
         if self.lexer_matchers:
             return self.lexer_matchers
-        else:
+        else:  # pragma: no cover
             raise ValueError(f"Lexing struct has not been set for dialect {self}")
 
     def patch_lexer_matchers(self, lexer_patch):
@@ -258,7 +307,7 @@ class Dialect:
         Used to edit the lexer of a sub-dialect.
         """
         buff = []
-        if not self.lexer_matchers:
+        if not self.lexer_matchers:  # pragma: no cover
             raise ValueError("Lexer struct must be defined before it can be patched!")
 
         # Make a new data struct for lookups
@@ -280,7 +329,7 @@ class Dialect:
         """
         buff = []
         found = False
-        if not self.lexer_matchers:
+        if not self.lexer_matchers:  # pragma: no cover
             raise ValueError("Lexer struct must be defined before it can be patched!")
 
         for elem in self.lexer_matchers:
@@ -292,9 +341,9 @@ class Dialect:
             else:
                 buff.append(elem)
 
-        if not found:
+        if not found:  # pragma: no cover
             raise ValueError(
-                "Lexer struct insert before '%s' failed because tag never found."
+                f"Lexer struct insert before '{before}' failed because tag never found."
             )
         # Overwrite with the buffer once we're done
         self.lexer_matchers = buff
