@@ -1,8 +1,15 @@
 """Linter class and helper classes."""
 
-# flake8: noqa: F401
-
 from sqlfluff.core.linter.common import RuleTuple, ParsedString, NoQaDirective
 from sqlfluff.core.linter.linted_file import LintedFile
 from sqlfluff.core.linter.linting_result import LintingResult
 from sqlfluff.core.linter.linter import Linter
+
+__all__ = (
+    "RuleTuple",
+    "ParsedString",
+    "NoQaDirective",
+    "LintedFile",
+    "LintingResult",
+    "Linter",
+)
