@@ -8,6 +8,7 @@ post linting.
 import os
 import logging
 import shutil
+import stat
 import tempfile
 from typing import (
     Any,
@@ -27,12 +28,12 @@ from sqlfluff.core.errors import (
     CheckTuple,
 )
 from sqlfluff.core.string_helpers import findall
-from sqlfluff.core.templaters import TemplatedFile
+from sqlfluff.core.templaters import TemplatedFile, RawFileSlice
 
 # Classes needed only for type checking
-from sqlfluff.core.parser.segments.base import BaseSegment, FixPatch
+from sqlfluff.core.parser.segments import BaseSegment, FixPatch
 
-from sqlfluff.core.linter.common import NoQaDirective, EnrichedFixPatch
+from sqlfluff.core.linter.common import NoQaDirective
 
 # Instantiate the linter logger
 linter_logger: logging.Logger = logging.getLogger("sqlfluff.linter")
@@ -53,24 +54,50 @@ class LintedFile(NamedTuple):
         """Make a list of check_tuples.
 
         This assumes that all the violations found are
-        linting violations (and therefore implement `check_tuple()`).
-        If they don't then this function raises that error.
+        linting violations. If they don't then this function
+        raises that error.
         """
         vs: List[CheckTuple] = []
         v: SQLLintError
         for v in self.get_violations():
-            if hasattr(v, "check_tuple"):
+            if isinstance(v, SQLLintError):
                 vs.append(v.check_tuple())
             elif raise_on_non_linting_violations:
                 raise v
         return vs
 
+    @staticmethod
+    def deduplicate_in_source_space(
+        violations: List[SQLBaseError],
+    ) -> List[SQLBaseError]:
+        """Removes duplicates in the source space.
+
+        This is useful for templated files with loops, where we'll
+        get a violation for each pass around the loop, but the user
+        only cares about it once and we're only going to fix it once.
+
+        By filtering them early we get a more a more helpful CLI
+        output *and* and more efficient fixing routine (by handling
+        fewer fixes).
+        """
+        new_violations = []
+        dedupe_buffer = set()
+        for v in violations:
+            signature = v.source_signature()
+            if signature not in dedupe_buffer:
+                new_violations.append(v)
+                dedupe_buffer.add(signature)
+            else:
+                linter_logger.debug("Removing duplicate source violation: %s", v)
+        return new_violations
+
     def get_violations(
         self,
         rules: Optional[Union[str, Tuple[str, ...]]] = None,
         types: Optional[Union[Type[SQLBaseError], Iterable[Type[SQLBaseError]]]] = None,
         filter_ignore: bool = True,
-        fixable: bool = None,
+        filter_warning: bool = True,
+        fixable: Optional[bool] = None,
     ) -> list:
         """Get a list of violations, respecting filters and ignore options.
 
@@ -104,6 +131,9 @@ class LintedFile(NamedTuple):
             # Ignore any rules in the ignore mask
             if self.ignore_mask:
                 violations = self.ignore_masked_violations(violations, self.ignore_mask)
+        # Filter warning violations
+        if filter_warning:
+            violations = [v for v in violations if not v.warning]
         return violations
 
     @staticmethod
@@ -203,9 +233,7 @@ class LintedFile(NamedTuple):
         return not any(self.get_violations(filter_ignore=True))
 
     @staticmethod
-    def _log_hints(
-        patch: Union[EnrichedFixPatch, FixPatch], templated_file: TemplatedFile
-    ):
+    def _log_hints(patch: FixPatch, templated_file: TemplatedFile):
         """Log hints for debugging during patch generation."""
         # This next bit is ALL FOR LOGGING AND DEBUGGING
         max_log_length = 10
@@ -267,44 +295,62 @@ class LintedFile(NamedTuple):
 
         original_source = self.templated_file.source_str
 
-        # Make sure no patches overlap and divide up the source file into slices.
-        # Any Template tags in the source file are off limits.
-        source_only_slices = self.templated_file.source_only_slices()
+        # Generate patches from the fixed tree. In the process we sort
+        # and deduplicate them so that the resultant list is in the
+        # the right order for the source file without any duplicates.
+        # TODO: Requires a mechanism for generating patches for source only
+        # fixes.
+        filtered_source_patches = self._generate_source_patches(
+            self.tree, self.templated_file
+        )
 
+        # Any Template tags in the source file are off limits, unless
+        # we're explicitly fixing the source file.
+        source_only_slices = self.templated_file.source_only_slices()
         linter_logger.debug("Source-only slices: %s", source_only_slices)
 
+        # We now slice up the file using the patches and any source only slices.
+        # This gives us regions to apply changes to.
+        # TODO: This is the last hurdle for source only fixes.
+        slice_buff = self._slice_source_file_using_patches(
+            filtered_source_patches, source_only_slices, self.templated_file.source_str
+        )
+
+        linter_logger.debug("Final slice buffer: %s", slice_buff)
+
+        # Iterate through the patches, building up the new string.
+        fixed_source_string = self._build_up_fixed_source_string(
+            slice_buff, filtered_source_patches, self.templated_file.source_str
+        )
+
+        # The success metric here is whether anything ACTUALLY changed.
+        return fixed_source_string, fixed_source_string != original_source
+
+    @classmethod
+    def _generate_source_patches(
+        cls, tree: BaseSegment, templated_file: TemplatedFile
+    ) -> List[FixPatch]:
+        """Use the fixed tree to generate source patches.
+
+        Importantly here we deduplicate and sort the patches
+        from their position in the templated file into their
+        intended order in the source file.
+        """
         # Iterate patches, filtering and translating as we go:
         linter_logger.debug("### Beginning Patch Iteration.")
         filtered_source_patches = []
         dedupe_buffer = []
         # We use enumerate so that we get an index for each patch. This is entirely
         # so when debugging logs we can find a given patch again!
-        patch: Union[EnrichedFixPatch, FixPatch]
-        for idx, patch in enumerate(
-            self.tree.iter_patches(templated_str=self.templated_file.templated_str)
-        ):
+        for idx, patch in enumerate(tree.iter_patches(templated_file=templated_file)):
             linter_logger.debug("  %s Yielded patch: %s", idx, patch)
-            self._log_hints(patch, self.templated_file)
-
-            # Attempt to convert to source space.
-            try:
-                source_slice = self.templated_file.templated_slice_to_source_slice(
-                    patch.templated_slice,
-                )
-            except ValueError:  # pragma: no cover
-                linter_logger.info(
-                    "      - Skipping. Source space Value Error. i.e. attempted "
-                    "insertion within templated section."
-                )
-                # If we try and slice within a templated section, then we may fail
-                # in which case, we should skip this patch.
-                continue
+            cls._log_hints(patch, templated_file)
 
             # Check for duplicates
-            dedupe_tuple = (source_slice, patch.fixed_raw)
-            if dedupe_tuple in dedupe_buffer:
+            if patch.dedupe_tuple() in dedupe_buffer:
                 linter_logger.info(
-                    "      - Skipping. Source space Duplicate: %s", dedupe_tuple
+                    "      - Skipping. Source space Duplicate: %s",
+                    patch.dedupe_tuple(),
                 )
                 continue
 
@@ -312,120 +358,109 @@ class LintedFile(NamedTuple):
             # or disrupt any templated sections.
             # The intent here is that unless explicitly stated, a fix should never
             # disrupt a templated section.
-            # NOTE: We rely here on the patches being sorted.
-            # TODO: Implement a mechanism for doing templated section fixes. For
-            # now it's just not allowed.
+            # NOTE: We rely here on the patches being generated in order.
+            # TODO: Implement a mechanism for doing templated section fixes. Given
+            # these patches are currently generated from fixed segments, there will
+            # likely need to be an entirely different mechanism
 
             # Get the affected raw slices.
-            local_raw_slices = self.templated_file.raw_slices_spanning_source_slice(
-                source_slice
+            local_raw_slices = templated_file.raw_slices_spanning_source_slice(
+                patch.source_slice
             )
             local_type_list = [slc.slice_type for slc in local_raw_slices]
 
-            enriched_patch = EnrichedFixPatch(
-                source_slice=source_slice,
-                templated_slice=patch.templated_slice,
-                patch_category=patch.patch_category,
-                fixed_raw=patch.fixed_raw,
-                templated_str=self.templated_file.templated_str[patch.templated_slice],
-                source_str=self.templated_file.source_str[source_slice],
-            )
-
             # Deal with the easy cases of 1) New code at end 2) only literals
             if not local_type_list or set(local_type_list) == {"literal"}:
                 linter_logger.info(
-                    "      * Keeping patch on new or literal-only section: %s",
-                    enriched_patch,
+                    "      * Keeping patch on new or literal-only section.",
                 )
-                filtered_source_patches.append(enriched_patch)
-                dedupe_buffer.append(enriched_patch.dedupe_tuple())
+                filtered_source_patches.append(patch)
+                dedupe_buffer.append(patch.dedupe_tuple())
+            # Handle the easy case of an explicit source fix
+            elif patch.patch_category == "source":
+                linter_logger.info(
+                    "      * Keeping explicit source fix patch.",
+                )
+                filtered_source_patches.append(patch)
+                dedupe_buffer.append(patch.dedupe_tuple())
             # Is it a zero length patch.
             elif (
-                enriched_patch.source_slice.start == enriched_patch.source_slice.stop
-                and enriched_patch.source_slice.start == local_raw_slices[0].source_idx
+                patch.source_slice.start == patch.source_slice.stop
+                and patch.source_slice.start == local_raw_slices[0].source_idx
             ):
                 linter_logger.info(
-                    "      * Keeping insertion patch on slice boundary: %s",
-                    enriched_patch,
-                )
-                filtered_source_patches.append(enriched_patch)
-                dedupe_buffer.append(enriched_patch.dedupe_tuple())
-            # If it's ONLY templated then we should skip it.
-            elif "literal" not in local_type_list:  # pragma: no cover
-                linter_logger.info(
-                    "      - Skipping patch over templated section: %s", enriched_patch
-                )
-            # If we span more than two slices then we should just skip it. Too Hard.
-            elif len(local_raw_slices) > 2:  # pragma: no cover
-                linter_logger.info(
-                    "      - Skipping patch over more than two raw slices: %s",
-                    enriched_patch,
-                )
-            # If it's an insertion (i.e. the string in the pre-fix template is '') then
-            # we won't be able to place it, so skip.
-            elif not enriched_patch.templated_str:  # pragma: no cover TODO?
-                linter_logger.info(
-                    "      - Skipping insertion patch in templated section: %s",
-                    enriched_patch,
-                )
-            # If the string from the templated version isn't in the source, then we
-            # can't fix it.
-            elif (
-                enriched_patch.templated_str not in enriched_patch.source_str
-            ):  # pragma: no cover TODO?
-                linter_logger.info(
-                    "      - Skipping edit patch on templated content: %s",
-                    enriched_patch,
+                    "      * Keeping insertion patch on slice boundary.",
                 )
+                filtered_source_patches.append(patch)
+                dedupe_buffer.append(patch.dedupe_tuple())
             else:  # pragma: no cover
+                # We've got a situation where the ends of our patch need to be
+                # more carefully mapped. Likely because we're greedily including
+                # a section of source templating with our fix and we need to work
+                # around it gracefully.
+
                 # Identify all the places the string appears in the source content.
-                positions = list(
-                    findall(enriched_patch.templated_str, enriched_patch.source_str)
-                )
+                positions = list(findall(patch.templated_str, patch.source_str))
                 if len(positions) != 1:
-                    linter_logger.debug(
+                    # NOTE: This section is not covered in tests. While we
+                    # don't have an example of it's use (we should), the
+                    # code after this relies on there being only one
+                    # instance found - so the safety check remains.
+                    linter_logger.debug(  # pragma: no cover
                         "        - Skipping edit patch on non-unique templated "
                         "content: %s",
-                        enriched_patch,
+                        patch,
                     )
-                    continue
+                    continue  # pragma: no cover
+
                 # We have a single occurrence of the thing we want to patch. This
                 # means we can use its position to place our patch.
-                new_source_slice = slice(  # pragma: no cover
-                    enriched_patch.source_slice.start + positions[0],
-                    enriched_patch.source_slice.start
-                    + positions[0]
-                    + len(enriched_patch.templated_str),
+                new_source_slice = slice(
+                    patch.source_slice.start + positions[0],
+                    patch.source_slice.start + positions[0] + len(patch.templated_str),
                 )
-                enriched_patch = EnrichedFixPatch(  # pragma: no cover
-                    source_slice=new_source_slice,
-                    templated_slice=enriched_patch.templated_slice,
-                    patch_category=enriched_patch.patch_category,
-                    fixed_raw=enriched_patch.fixed_raw,
-                    templated_str=enriched_patch.templated_str,
-                    source_str=enriched_patch.source_str,
-                )
-                linter_logger.debug(  # pragma: no cover
+                linter_logger.debug(
                     "      * Keeping Tricky Case. Positions: %s, New Slice: %s, "
                     "Patch: %s",
                     positions,
                     new_source_slice,
-                    enriched_patch,
+                    patch,
                 )
-                filtered_source_patches.append(enriched_patch)  # pragma: no cover
-                dedupe_buffer.append(enriched_patch.dedupe_tuple())  # pragma: no cover
-                continue  # pragma: no cover
+                patch.source_slice = new_source_slice
+                filtered_source_patches.append(patch)
+                dedupe_buffer.append(patch.dedupe_tuple())
+                continue
 
         # Sort the patches before building up the file.
-        filtered_source_patches = sorted(
-            filtered_source_patches, key=lambda x: x.source_slice.start
-        )
+        return sorted(filtered_source_patches, key=lambda x: x.source_slice.start)
+
+    @staticmethod
+    def _slice_source_file_using_patches(
+        source_patches: List[FixPatch],
+        source_only_slices: List[RawFileSlice],
+        raw_source_string: str,
+    ) -> List[slice]:
+        """Use patches to safely slice up the file before fixing.
+
+        This uses source only slices to avoid overwriting sections
+        of templated code in the source file (when we don't want to).
+
+        We assume that the source patches have already been
+        sorted and deduplicated. Sorting is important. If the slices
+        aren't sorted then this function will miss chunks.
+        If there are overlaps or duplicates then this function
+        may produce strange results.
+        """
         # We now slice up the file using the patches and any source only slices.
         # This gives us regions to apply changes to.
         slice_buff = []
         source_idx = 0
-        for patch in filtered_source_patches:
+        for patch in source_patches:
             # Are there templated slices at or before the start of this patch?
+            # TODO: We'll need to explicit handling for template fixes here, because
+            # they ARE source only slices. If we can get handling to work properly
+            # here then this is the last hurdle and it will flow through
+            # smoothly from here.
             while (
                 source_only_slices
                 and source_only_slices[0].source_idx < patch.source_slice.start
@@ -438,6 +473,20 @@ class LintedFile(NamedTuple):
                 slice_buff.append(next_so_slice)
                 source_idx = next_so_slice.stop
 
+            # Does this patch cover the next source-only slice directly?
+            if (
+                source_only_slices
+                and patch.source_slice == source_only_slices[0].source_slice()
+            ):
+                linter_logger.info(
+                    "Removing next source only slice from the stack because it "
+                    "covers the same area of source file as the current patch: %s %s",
+                    source_only_slices[0],
+                    patch,
+                )
+                # If it does, remove it so that we don't duplicate it.
+                source_only_slices.pop(0)
+
             # Is there a gap between current position and this patch?
             if patch.source_slice.start > source_idx:
                 # Add a slice up to this patch.
@@ -457,16 +506,30 @@ class LintedFile(NamedTuple):
             slice_buff.append(patch.source_slice)
             source_idx = patch.source_slice.stop
         # Add a tail slice.
-        if source_idx < len(self.templated_file.source_str):
-            slice_buff.append(slice(source_idx, len(self.templated_file.source_str)))
+        if source_idx < len(raw_source_string):
+            slice_buff.append(slice(source_idx, len(raw_source_string)))
 
-        linter_logger.debug("Final slice buffer: %s", slice_buff)
+        return slice_buff
 
+    @staticmethod
+    def _build_up_fixed_source_string(
+        source_file_slices: List[slice],
+        source_patches: List[FixPatch],
+        raw_source_string: str,
+    ) -> str:
+        """Use patches and raw file to fix the source file.
+
+        This assumes that patches and slices have already
+        been coordinated. If they haven't then this will
+        fail because we rely on patches having a corresponding
+        slice of exactly the right file in the list of file
+        slices.
+        """
         # Iterate through the patches, building up the new string.
         str_buff = ""
-        for source_slice in slice_buff:
+        for source_slice in source_file_slices:
             # Is it one in the patch buffer:
-            for patch in filtered_source_patches:
+            for patch in source_patches:
                 if patch.source_slice == source_slice:
                     # Use the patched version
                     linter_logger.debug(
@@ -483,12 +546,10 @@ class LintedFile(NamedTuple):
                 linter_logger.debug(
                     "Appending Raw:                    %s     %r",
                     source_slice,
-                    self.templated_file.source_str[source_slice],
+                    raw_source_string[source_slice],
                 )
-                str_buff += self.templated_file.source_str[source_slice]
-
-        # The success metric here is whether anything ACTUALLY changed.
-        return str_buff, str_buff != original_source
+                str_buff += raw_source_string[source_slice]
+        return str_buff
 
     def persist_tree(self, suffix: str = "") -> bool:
         """Persist changes to the given path."""
@@ -500,24 +561,39 @@ class LintedFile(NamedTuple):
             if suffix:
                 root, ext = os.path.splitext(fname)
                 fname = root + suffix + ext
-            self._safe_create_replace_file(fname, write_buff, self.encoding)
+            self._safe_create_replace_file(self.path, fname, write_buff, self.encoding)
         return success
 
     @staticmethod
-    def _safe_create_replace_file(fname, write_buff, encoding):
+    def _safe_create_replace_file(
+        input_path: str, output_path: str, write_buff: str, encoding: str
+    ):
         # Write to a temporary file first, so in case of encoding or other
         # issues, we don't delete or corrupt the user's existing file.
-        dirname, basename = os.path.split(fname)
+
+        # Get file mode (i.e. permissions) on existing file. We'll preserve the
+        # same permissions on the output file.
+        mode = None
+        try:
+            status = os.stat(input_path)
+        except FileNotFoundError:
+            pass
+        else:
+            if stat.S_ISREG(status.st_mode):
+                mode = stat.S_IMODE(status.st_mode)
+        dirname, basename = os.path.split(output_path)
         with tempfile.NamedTemporaryFile(
             mode="w",
             encoding=encoding,
             prefix=basename,
             dir=dirname,
-            suffix=os.path.splitext(fname)[1],
+            suffix=os.path.splitext(output_path)[1],
             delete=False,
         ) as tmp:
             tmp.file.write(write_buff)
             tmp.flush()
             os.fsync(tmp.fileno())
         # Once the temp file is safely written, replace the existing file.
-        shutil.move(tmp.name, fname)
+        if mode is not None:
+            os.chmod(tmp.name, mode)
+        shutil.move(tmp.name, output_path)
