@@ -6,13 +6,15 @@ import time
 import logging
 from typing import (
     Any,
-    List,
-    Sequence,
-    Optional,
-    Tuple,
-    cast,
     Iterable,
     Iterator,
+    List,
+    Optional,
+    Sequence,
+    Set,
+    Tuple,
+    Type,
+    cast,
 )
 
 import pathspec
@@ -24,19 +26,21 @@ from sqlfluff.core.errors import (
     SQLLexError,
     SQLLintError,
     SQLParseError,
-    SQLTemplaterSkipFile,
+    SQLFluffSkipFile,
+    SQLFluffUserError,
 )
 from sqlfluff.core.parser import Lexer, Parser, RegexLexer
 from sqlfluff.core.file_helpers import get_encoding
 from sqlfluff.core.templaters import TemplatedFile
 from sqlfluff.core.rules import get_ruleset
+from sqlfluff.core.rules.doc_decorators import is_fix_compatible
 from sqlfluff.core.config import FluffConfig, ConfigLoader, progress_bar_configuration
 
 # Classes needed only for type checking
-from sqlfluff.core.parser.segments.base import BaseSegment
+from sqlfluff.core.parser.segments.base import BaseSegment, SourceFix
 from sqlfluff.core.parser.segments.meta import MetaSegment
 from sqlfluff.core.parser.segments.raw import RawSegment
-from sqlfluff.core.rules.base import BaseRule
+from sqlfluff.core.rules import BaseRule
 
 from sqlfluff.core.linter.common import (
     RuleTuple,
@@ -67,7 +71,7 @@ class Linter:
         formatter: Any = None,
         dialect: Optional[str] = None,
         rules: Optional[List[str]] = None,
-        user_rules: Optional[List[BaseRule]] = None,
+        user_rules: Optional[List[Type[BaseRule]]] = None,
         exclude_rules: Optional[List[str]] = None,
     ) -> None:
         # Store the config object
@@ -76,6 +80,10 @@ class Linter:
             dialect=dialect,
             rules=rules,
             exclude_rules=exclude_rules,
+            # Don't require a dialect to be provided yet. Defer this until we
+            # are actually linting something, since the directory we are linting
+            # from may provide additional configuration, including a dialect.
+            require_dialect=False,
         )
         # Get the dialect and templater
         self.dialect = self.config.get("dialect_obj")
@@ -103,12 +111,25 @@ class Linter:
     # These are the building blocks of the linting process.
 
     @staticmethod
-    def _load_raw_file_and_config(
+    def load_raw_file_and_config(
         fname: str, root_config: FluffConfig
     ) -> Tuple[str, FluffConfig, str]:
         """Load a raw file and the associated config."""
         file_config = root_config.make_child_from_path(fname)
         encoding = get_encoding(fname=fname, config=file_config)
+        # Check file size before loading.
+        limit = file_config.get("large_file_skip_byte_limit")
+        if limit:
+            # Get the file size
+            file_size = os.path.getsize(fname)
+            if file_size > limit:
+                raise SQLFluffSkipFile(
+                    f"Length of file {fname!r} is {file_size} bytes which is over "
+                    f"the limit of {limit} bytes. Skipping to avoid parser lock. "
+                    "Users can increase this limit in their config by setting the "
+                    "'large_file_skip_byte_limit' value, or disable by setting it "
+                    "to zero."
+                )
         with open(fname, encoding=encoding, errors="backslashreplace") as target_file:
             raw_file = target_file.read()
         # Scan the raw file for config commands.
@@ -142,7 +163,7 @@ class Linter:
             linter_logger.info(
                 "Lexed tokens: %s", [seg.raw for seg in tokens] if tokens else None
             )
-        except SQLLexError as err:
+        except SQLLexError as err:  # pragma: no cover
             linter_logger.info("LEXING FAILED! (%s): %s", templated_file.fname, err)
             violations.append(err)
             return None, violations, config
@@ -329,6 +350,12 @@ class Linter:
                 result.append(e)
         return result
 
+    @staticmethod
+    def _report_conflicting_fixes_same_anchor(message: str):  # pragma: no cover
+        # This function exists primarily in order to let us monkeypatch it at
+        # runtime (replacing it with a function that raises an exception).
+        linter_logger.critical(message)
+
     @staticmethod
     def _warn_unfixable(code: str):
         linter_logger.warning(
@@ -410,7 +437,7 @@ class Linter:
         ignore_buff: List[NoQaDirective] = []
         violations: List[SQLBaseError] = []
         for comment in tree.recursive_crawl("comment"):
-            if comment.name == "inline_comment":
+            if comment.is_type("inline_comment"):
                 ignore_entry = cls.extract_ignore_from_comment(comment, rule_codes)
                 if isinstance(ignore_entry, SQLParseError):
                     violations.append(ignore_entry)
@@ -460,12 +487,15 @@ class Linter:
         formatter: Any = None,
     ) -> Tuple[BaseSegment, List[SQLBaseError], List[NoQaDirective]]:
         """Lint and optionally fix a tree object."""
-        # Keep track of the linting errors
-        all_linting_errors = []
+        # Keep track of the linting errors on the very first linter pass. The
+        # list of issues output by "lint" and "fix" only includes issues present
+        # in the initial SQL code, EXCLUDING any issues that may be created by
+        # the fixes themselves.
+        initial_linting_errors = []
         # A placeholder for the fixes we had on the previous loop
         last_fixes = None
         # Keep a set of previous versions to catch infinite loops.
-        previous_versions = {tree.raw}
+        previous_versions: Set[Tuple[str, Tuple[SourceFix, ...]]] = {(tree.raw, ())}
 
         # If we are fixing then we want to loop up to the runaway_limit, otherwise just
         # once for linting.
@@ -479,95 +509,169 @@ class Linter:
         if not config.get("disable_noqa"):
             rule_codes = [r.code for r in rule_set]
             ignore_buff, ivs = cls.extract_ignore_mask_tree(tree, rule_codes)
-            all_linting_errors += ivs
+            initial_linting_errors += ivs
         else:
             ignore_buff = []
 
         save_tree = tree
-        for loop in range(loop_limit):
-            changed = False
+        # There are two phases of rule running.
+        # 1. The main loop is for most rules. These rules are assumed to
+        # interact and cause a cascade of fixes requiring multiple passes.
+        # These are run the `runaway_limit` number of times (default 10).
+        # 2. The post loop is for post-processing rules, not expected to trigger
+        # any downstream rules, e.g. capitalization fixes. They are run on the
+        # first loop and then twice at the end (once to fix, and once again to
+        # check result of fixes), but not in the intervening loops.
+        phases = ["main"]
+        if fix:
+            phases.append("post")
+        for phase in phases:
+            if len(phases) > 1:
+                rules_this_phase = [
+                    rule for rule in rule_set if rule.lint_phase == phase
+                ]
+            else:
+                rules_this_phase = rule_set
+            for loop in range(loop_limit if phase == "main" else 2):
 
-            progress_bar_crawler = tqdm(
-                rule_set,
-                desc="lint by rules",
-                leave=False,
-                disable=progress_bar_configuration.disable_progress_bar,
-            )
+                def is_first_linter_pass():
+                    return phase == phases[0] and loop == 0
 
-            for crawler in progress_bar_crawler:
-                progress_bar_crawler.set_description(f"rule {crawler.code}")
-
-                # fixes should be a dict {} with keys edit, delete, create
-                # delete is just a list of segments to delete
-                # edit and create are list of tuples. The first element is the
-                # "anchor", the segment to look for either to edit or to insert BEFORE.
-                # The second is the element to insert or create.
-                linting_errors, _, fixes, _ = crawler.crawl(
-                    tree,
-                    ignore_mask=ignore_buff,
-                    dialect=config.get("dialect_obj"),
-                    fname=fname,
-                    templated_file=templated_file,
-                )
-                all_linting_errors += linting_errors
-
-                if fix and fixes:
-                    linter_logger.info(f"Applying Fixes [{crawler.code}]: {fixes}")
-                    # Do some sanity checks on the fixes before applying.
-                    if fixes == last_fixes:  # pragma: no cover
-                        cls._warn_unfixable(crawler.code)
-                    else:
-                        last_fixes = fixes
-                        new_tree, _ = tree.apply_fixes(fixes)
-                        # Check for infinite loops
-                        if new_tree.raw not in previous_versions:
-                            # We've not seen this version of the file so far. Continue.
-                            tree = new_tree
-                            previous_versions.add(tree.raw)
-                            changed = True
-                            continue
-                        else:
-                            # Applying these fixes took us back to a state which we've
-                            # seen before. Abort.
-                            cls._warn_unfixable(crawler.code)
-
-            if loop == 0:
-                # Keep track of initial errors for reporting.
-                initial_linting_errors = all_linting_errors.copy()
-
-            if fix and not changed:
-                # We did not change the file. Either the file is clean (no fixes), or
-                # any fixes which are present will take us back to a previous state.
+                # Additional newlines are to assist in scanning linting loops
+                # during debugging.
                 linter_logger.info(
-                    f"Fix loop complete. Stability achieved after {loop}/{loop_limit} "
-                    "loops."
+                    f"\n\nEntering linter phase {phase}, loop {loop+1}/{loop_limit}\n"
                 )
-                break
-        else:
-            if fix:
-                # The linter loop hit the limit before reaching a stable point
-                # (i.e. free of lint errors). If this happens, it's usually
-                # because one or more rules produced fixes which did not address
-                # the original issue **or** created new issues.
-                linter_logger.warning(f"Loop limit on fixes reached [{loop_limit}].")
+                changed = False
 
-                # Discard any fixes for the linting errors, since they caused a
-                # loop. IMPORTANT: By doing this, we are telling SQLFluff that
-                # these linting errors are "unfixable". This is important,
-                # because when "sqlfluff fix" encounters unfixable lint errors,
-                # it exits with a "failure" exit code, which is exactly what we
-                # want in this situation. (Reason: Although this is more of an
-                # internal SQLFluff issue, users deserve to know about it,
-                # because it means their file(s) weren't fixed.
-                for violation in initial_linting_errors:
-                    if isinstance(violation, SQLLintError):
-                        violation.fixes = []
+                if is_first_linter_pass():
+                    # In order to compute initial_linting_errors correctly, need
+                    # to run all rules on the first loop of the main phase.
+                    rules_this_phase = rule_set
+                progress_bar_crawler = tqdm(
+                    rules_this_phase,
+                    desc="lint by rules",
+                    leave=False,
+                    disable=progress_bar_configuration.disable_progress_bar,
+                )
 
-                # Return the original parse tree, before any fixes were applied.
-                # Reason: When the linter hits the loop limit, the file is often
-                # messy, e.g. some of the fixes were applied repeatedly, possibly
-                # other weird things. We don't want the user to see this junk!
-                return save_tree, initial_linting_errors, ignore_buff
+                for crawler in progress_bar_crawler:
+                    # Performance: After first loop pass, skip rules that don't
+                    # do fixes. Any results returned won't be seen by the user
+                    # anyway (linting errors ADDED by rules changing SQL, are
+                    # not reported back to the user - only initial linting errors),
+                    # so there's absolutely no reason to run them.
+                    if (
+                        fix
+                        and not is_first_linter_pass()
+                        and not is_fix_compatible(crawler)
+                    ):
+                        continue
+
+                    progress_bar_crawler.set_description(f"rule {crawler.code}")
+
+                    # fixes should be a dict {} with keys edit, delete, create
+                    # delete is just a list of segments to delete
+                    # edit and create are list of tuples. The first element is
+                    # the "anchor", the segment to look for either to edit or to
+                    # insert BEFORE. The second is the element to insert or create.
+                    linting_errors, _, fixes, _ = crawler.crawl(
+                        tree,
+                        dialect=config.get("dialect_obj"),
+                        fix=fix,
+                        templated_file=templated_file,
+                        ignore_mask=ignore_buff,
+                        fname=fname,
+                        config=config,
+                    )
+                    if is_first_linter_pass():
+                        initial_linting_errors += linting_errors
+
+                    if fix and fixes:
+                        linter_logger.info(f"Applying Fixes [{crawler.code}]: {fixes}")
+                        # Do some sanity checks on the fixes before applying.
+                        anchor_info = BaseSegment.compute_anchor_edit_info(fixes)
+                        if any(
+                            not info.is_valid for info in anchor_info.values()
+                        ):  # pragma: no cover
+                            message = (
+                                f"Rule {crawler.code} returned conflicting "
+                                "fixes with the same anchor. This is only "
+                                "supported for create_before+create_after, so "
+                                f"the fixes will not be applied. {fixes!r}"
+                            )
+                            cls._report_conflicting_fixes_same_anchor(message)
+                            for lint_result in linting_errors:
+                                lint_result.fixes = []
+                        elif fixes == last_fixes:  # pragma: no cover
+                            # If we generate the same fixes two times in a row,
+                            # that means we're in a loop, and we want to stop.
+                            # (Fixes should address issues, hence different
+                            # and/or fewer fixes next time.)
+                            cls._warn_unfixable(crawler.code)
+                        else:
+                            # This is the happy path. We have fixes, now we want to
+                            # apply them.
+                            last_fixes = fixes
+                            new_tree, _, _ = tree.apply_fixes(
+                                config.get("dialect_obj"), crawler.code, anchor_info
+                            )
+                            # Check for infinite loops. We use a combination of the
+                            # fixed templated file and the list of source fixes to
+                            # apply.
+                            loop_check_tuple = (
+                                new_tree.raw,
+                                tuple(new_tree.source_fixes),
+                            )
+                            if loop_check_tuple not in previous_versions:
+                                # We've not seen this version of the file so
+                                # far. Continue.
+                                tree = new_tree
+                                previous_versions.add(loop_check_tuple)
+                                changed = True
+                                continue
+                            else:
+                                # Applying these fixes took us back to a state
+                                # which we've seen before. We're in a loop, so
+                                # we want to stop.
+                                cls._warn_unfixable(crawler.code)
+
+                if fix and not changed:
+                    # We did not change the file. Either the file is clean (no
+                    # fixes), or any fixes which are present will take us back
+                    # to a previous state.
+                    linter_logger.info(
+                        f"Fix loop complete for {phase} phase. Stability "
+                        f"achieved after {loop}/{loop_limit} loops."
+                    )
+                    break
+            else:
+                if fix:
+                    # The linter loop hit the limit before reaching a stable point
+                    # (i.e. free of lint errors). If this happens, it's usually
+                    # because one or more rules produced fixes which did not address
+                    # the original issue **or** created new issues.
+                    linter_logger.warning(
+                        f"Loop limit on fixes reached [{loop_limit}]."
+                    )
+
+                    # Discard any fixes for the linting errors, since they caused a
+                    # loop. IMPORTANT: By doing this, we are telling SQLFluff that
+                    # these linting errors are "unfixable". This is important,
+                    # because when "sqlfluff fix" encounters unfixable lint errors,
+                    # it exits with a "failure" exit code, which is exactly what we
+                    # want in this situation. (Reason: Although this is more of an
+                    # internal SQLFluff issue, users deserve to know about it,
+                    # because it means their file(s) weren't fixed.
+                    for violation in initial_linting_errors:
+                        if isinstance(violation, SQLLintError):
+                            violation.fixes = []
+
+                    # Return the original parse tree, before any fixes were applied.
+                    # Reason: When the linter hits the loop limit, the file is often
+                    # messy, e.g. some of the fixes were applied repeatedly, possibly
+                    # other weird things. We don't want the user to see this junk!
+                    return save_tree, initial_linting_errors, ignore_buff
 
         if config.get("ignore_templated_areas", default=True):
             initial_linting_errors = cls.remove_templated_errors(initial_linting_errors)
@@ -628,10 +732,12 @@ class Linter:
         # We process the ignore config here if appropriate
         for violation in violations:
             violation.ignore_if_in(parsed.config.get("ignore"))
+            violation.warning_if_in(parsed.config.get("warnings"))
 
         linted_file = LintedFile(
             parsed.fname,
-            violations,
+            # Deduplicate violations
+            LintedFile.deduplicate_in_source_space(violations),
             time_dict,
             tree,
             ignore_mask=ignore_buff,
@@ -646,11 +752,11 @@ class Linter:
             )
 
         # Safety flag for unset dialects
-        if parsed.config.get("dialect") == "ansi" and linted_file.get_violations(
+        if linted_file.get_violations(
             fixable=True if fix else None, types=SQLParseError
         ):
             if formatter:  # pragma: no cover TODO?
-                formatter.dispatch_dialect_warning()
+                formatter.dispatch_dialect_warning(parsed.config.get("dialect"))
 
         return linted_file
 
@@ -690,6 +796,11 @@ class Linter:
         # we want consistent mapping between the raw and templated slices.
         in_str = self._normalise_newlines(in_str)
 
+        # Since Linter.__init__() does not require a dialect to be specified,
+        # check for one now. (We're processing a string, not a file, so we're
+        # not going to pick up a .sqlfluff or other config file to provide a
+        # missing dialect at this point.)
+        config.verify_dialect_specified()
         if not config.get("templater_obj") == self.templater:
             linter_logger.warning(
                 (
@@ -705,7 +816,7 @@ class Linter:
             templated_file, templater_violations = self.templater.process(
                 in_str=in_str, fname=fname, config=config, formatter=self.formatter
             )
-        except SQLTemplaterSkipFile as s:  # pragma: no cover
+        except SQLFluffSkipFile as s:  # pragma: no cover
             linter_logger.warning(str(s))
             templated_file = None
             templater_violations = []
@@ -729,7 +840,7 @@ class Linter:
     def render_file(self, fname: str, root_config: FluffConfig) -> RenderedFile:
         """Load and render a file with relevant config."""
         # Load the raw file.
-        raw_file, config, encoding = self._load_raw_file_and_config(fname, root_config)
+        raw_file, config, encoding = self.load_raw_file_and_config(fname, root_config)
         # Render the file
         return self.render_string(raw_file, fname, config, encoding)
 
@@ -861,7 +972,9 @@ class Linter:
             if ignore_non_existent_files:
                 return []
             else:
-                raise OSError("Specified path does not exist")
+                raise SQLFluffUserError(
+                    f"Specified path does not exist. Check it/they exist(s): {path}."
+                )
 
         # Files referred to exactly are also ignored if
         # matched, but we warn the users when that happens
@@ -907,23 +1020,26 @@ class Linter:
                     with open(fpath) as fh:
                         spec = pathspec.PathSpec.from_lines("gitwildmatch", fh)
                         ignores[dirpath] = spec
-                    # We don't need to process the ignore file any futher
+                    # We don't need to process the ignore file any further
                     continue
 
                 # We won't purge files *here* because there's an edge case
                 # that the ignore file is processed after the sql file.
 
                 # Scan for remaining files
-                for ext in self.config.get("sql_file_exts", default=".sql").split(","):
+                for ext in (
+                    self.config.get("sql_file_exts", default=".sql").lower().split(",")
+                ):
                     # is it a sql file?
-                    if fname.endswith(ext):
+                    if fname.lower().endswith(ext):
                         buffer.append(fpath)
 
         if not ignore_files:
             return sorted(buffer)
 
         # Check the buffer for ignore items and normalise the rest.
-        filtered_buffer = []
+        # It's a set, so we can do natural deduplication.
+        filtered_buffer = set()
 
         for fpath in buffer:
             abs_fpath = os.path.abspath(fpath)
@@ -950,9 +1066,18 @@ class Linter:
                         )
                     break
             else:
-                filtered_buffer.append(os.path.normpath(fpath))
+                npath = os.path.normpath(fpath)
+                # For debugging, log if we already have the file.
+                if npath in filtered_buffer:
+                    linter_logger.debug(  # pragma: no cover
+                        "Developer Warning: Path crawler attempted to "
+                        "requeue the same file twice. %s is already in "
+                        "filtered buffer.",
+                        npath,
+                    )
+                filtered_buffer.add(npath)
 
-        # Return
+        # Return a sorted list
         return sorted(filtered_buffer)
 
     def lint_string_wrapped(
@@ -975,57 +1100,12 @@ class Linter:
         fix: bool = False,
         ignore_non_existent_files: bool = False,
         ignore_files: bool = True,
-        processes: int = 1,
+        processes: Optional[int] = None,
     ) -> LintedDir:
         """Lint a path."""
-        linted_path = LintedDir(path)
-        if self.formatter:
-            self.formatter.dispatch_path(path)
-        fnames = list(
-            self.paths_from_path(
-                path,
-                ignore_non_existent_files=ignore_non_existent_files,
-                ignore_files=ignore_files,
-            )
-        )
-
-        # to avoid circular import
-        from sqlfluff.core.linter.runner import get_runner
-
-        runner = get_runner(
-            self,
-            self.config,
-            processes=processes,
-            allow_process_parallelism=self.allow_process_parallelism,
-        )
-
-        # Show files progress bar only when there is more than one.
-        files_count = len(fnames)
-        progress_bar_files = tqdm(
-            total=files_count,
-            desc=f"file {os.path.basename(fnames[0] if fnames else '')}",
-            leave=False,
-            disable=files_count <= 1 or progress_bar_configuration.disable_progress_bar,
-        )
-
-        for i, linted_file in enumerate(runner.run(fnames, fix), start=1):
-            linted_path.add(linted_file)
-            # If any fatal errors, then stop iteration.
-            if any(v.fatal for v in linted_file.violations):  # pragma: no cover
-                linter_logger.error("Fatal linting error. Halting further linting.")
-                break
-
-            # Progress bar for files is rendered only when there is more than one file.
-            # Additionally as it's updated after each loop, we need to get file name
-            # from the next loop. This is why `enumerate` starts with `1` and there
-            # is `i < len` to not exceed files list length.
-            progress_bar_files.update(n=1)
-            if i < len(fnames):
-                progress_bar_files.set_description(
-                    f"file {os.path.basename(fnames[i])}"
-                )
-
-        return linted_path
+        return self.lint_paths(
+            (path,), fix, ignore_non_existent_files, ignore_files, processes
+        ).paths[0]
 
     def lint_paths(
         self,
@@ -1033,39 +1113,69 @@ class Linter:
         fix: bool = False,
         ignore_non_existent_files: bool = False,
         ignore_files: bool = True,
-        processes: int = 1,
+        processes: Optional[int] = None,
     ) -> LintingResult:
         """Lint an iterable of paths."""
-        paths_count = len(paths)
-
         # If no paths specified - assume local
-        if not paths_count:  # pragma: no cover
+        if not paths:  # pragma: no cover
             paths = (os.getcwd(),)
         # Set up the result to hold what we get back
         result = LintingResult()
 
-        progress_bar_paths = tqdm(
-            total=paths_count,
-            desc="path",
-            leave=False,
-            disable=paths_count <= 1 or progress_bar_configuration.disable_progress_bar,
-        )
+        expanded_paths: List[str] = []
+        expanded_path_to_linted_dir = {}
         for path in paths:
-            progress_bar_paths.set_description(f"path {path}")
+            linted_dir = LintedDir(path)
+            result.add(linted_dir)
+            for fname in self.paths_from_path(
+                path,
+                ignore_non_existent_files=ignore_non_existent_files,
+                ignore_files=ignore_files,
+            ):
+                expanded_paths.append(fname)
+                expanded_path_to_linted_dir[fname] = linted_dir
 
-            # Iterate through files recursively in the specified directory (if it's a
-            # directory) or read the file directly if it's not
-            result.add(
-                self.lint_path(
-                    path,
-                    fix=fix,
-                    ignore_non_existent_files=ignore_non_existent_files,
-                    ignore_files=ignore_files,
-                    processes=processes,
-                )
-            )
+        files_count = len(expanded_paths)
+        if processes is None:
+            processes = self.config.get("processes", default=1)
 
-            progress_bar_paths.update(1)
+        # to avoid circular import
+        from sqlfluff.core.linter.runner import get_runner
+
+        runner, effective_processes = get_runner(
+            self,
+            self.config,
+            processes=processes,
+            allow_process_parallelism=self.allow_process_parallelism,
+        )
+
+        if self.formatter and effective_processes != 1:
+            self.formatter.dispatch_processing_header(effective_processes)
+
+        # Show files progress bar only when there is more than one.
+        first_path = expanded_paths[0] if expanded_paths else ""
+        progress_bar_files = tqdm(
+            total=files_count,
+            desc=f"file {first_path}",
+            leave=False,
+            disable=files_count <= 1 or progress_bar_configuration.disable_progress_bar,
+        )
+
+        for i, linted_file in enumerate(runner.run(expanded_paths, fix), start=1):
+            linted_dir = expanded_path_to_linted_dir[linted_file.path]
+            linted_dir.add(linted_file)
+            # If any fatal errors, then stop iteration.
+            if any(v.fatal for v in linted_file.violations):  # pragma: no cover
+                linter_logger.error("Fatal linting error. Halting further linting.")
+                break
+
+            # Progress bar for files is rendered only when there is more than one file.
+            # Additionally, as it's updated after each loop, we need to get file name
+            # from the next loop. This is why `enumerate` starts with `1` and there
+            # is `i < len` to not exceed files list length.
+            progress_bar_files.update(n=1)
+            if i < len(expanded_paths):
+                progress_bar_files.set_description(f"file {expanded_paths[i]}")
 
         result.stop_timer()
         return result
@@ -1084,9 +1194,13 @@ class Linter:
             if self.formatter:
                 self.formatter.dispatch_path(path)
             # Load the file with the config and yield the result.
-            raw_file, config, encoding = self._load_raw_file_and_config(
-                fname, self.config
-            )
+            try:
+                raw_file, config, encoding = self.load_raw_file_and_config(
+                    fname, self.config
+                )
+            except SQLFluffSkipFile as s:
+                linter_logger.warning(str(s))
+                continue
             yield self.parse_string(
                 raw_file,
                 fname=fname,
