@@ -1,5 +1,6 @@
 """Defines the linter class."""
 
+import csv
 import time
 from typing import (
     Any,
@@ -7,20 +8,24 @@ from typing import (
     List,
     Optional,
     overload,
+    Tuple,
+    Union,
 )
 from typing_extensions import Literal
 
+from sqlfluff.cli import EXIT_FAIL, EXIT_SUCCESS
 
 from sqlfluff.core.errors import (
     CheckTuple,
+    SQLLintError,
+    SQLParseError,
+    SQLTemplaterError,
 )
 
 from sqlfluff.core.timing import TimingSummary
 
 # Classes needed only for type checking
 from sqlfluff.core.parser.segments.base import BaseSegment
-
-
 from sqlfluff.core.linter.linted_dir import LintedDir
 
 
@@ -59,21 +64,27 @@ class LintingResult:
         self.total_time = time.monotonic() - self._start_time
 
     @overload
-    def check_tuples(self, by_path: Literal[False]) -> List[CheckTuple]:
+    def check_tuples(
+        self, by_path: Literal[False]
+    ) -> List[CheckTuple]:  # pragma: no cover
         """Return a List of CheckTuples when by_path is False."""
         ...
 
     @overload
-    def check_tuples(self, by_path: Literal[True]) -> Dict[LintedDir, List[CheckTuple]]:
+    def check_tuples(
+        self, by_path: Literal[True]
+    ) -> Dict[LintedDir, List[CheckTuple]]:  # pragma: no cover
         """Return a Dict of LintedDir and CheckTuples when by_path is True."""
         ...
 
     @overload
-    def check_tuples(self, by_path: bool = False):
+    def check_tuples(self, by_path: bool = False):  # pragma: no cover
         """Default overload method."""
         ...
 
-    def check_tuples(self, by_path=False):
+    def check_tuples(
+        self, by_path=False
+    ) -> Union[List[CheckTuple], Dict[LintedDir, List[CheckTuple]]]:
         """Fetch all check_tuples from all contained `LintedDir` objects.
 
         Args:
@@ -106,7 +117,9 @@ class LintingResult:
 
     def violation_dict(self, **kwargs):
         """Return a dict of paths and violations."""
-        return self.combine_dicts(path.violation_dict(**kwargs) for path in self.paths)
+        return self.combine_dicts(
+            *(path.violation_dict(**kwargs) for path in self.paths)
+        )
 
     def stats(self) -> Dict[str, Any]:
         """Return a stats dictionary of this result."""
@@ -123,7 +136,9 @@ class LintingResult:
             all_stats["unclean rate"] = 0
         all_stats["clean files"] = all_stats["clean"]
         all_stats["unclean files"] = all_stats["unclean"]
-        all_stats["exit code"] = 65 if all_stats["violations"] > 0 else 0
+        all_stats["exit code"] = (
+            EXIT_FAIL if all_stats["violations"] > 0 else EXIT_SUCCESS
+        )
         all_stats["status"] = "FAIL" if all_stats["violations"] > 0 else "PASS"
         return all_stats
 
@@ -135,12 +150,56 @@ class LintingResult:
                 timing.add(file.time_dict)
         return timing.summary()
 
+    def persist_timing_records(self, filename):
+        """Persist the timing records as a csv to external analysis."""
+        meta_fields = [
+            "path",
+            "source_chars",
+            "templated_chars",
+            "segments",
+            "raw_segments",
+        ]
+        timing_fields = ["templating", "lexing", "parsing", "linting"]
+        with open(filename, "w", newline="") as f:
+            writer = csv.DictWriter(f, fieldnames=meta_fields + timing_fields)
+
+            writer.writeheader()
+
+            for dir in self.paths:
+                for file in dir.files:
+                    writer.writerow(
+                        {
+                            "path": file.path,
+                            "source_chars": (
+                                len(file.templated_file.source_str)
+                                if file.templated_file
+                                else ""
+                            ),
+                            "templated_chars": (
+                                len(file.templated_file.templated_str)
+                                if file.templated_file
+                                else ""
+                            ),
+                            "segments": (
+                                file.tree.count_segments(raw_only=False)
+                                if file.tree
+                                else ""
+                            ),
+                            "raw_segments": (
+                                file.tree.count_segments(raw_only=True)
+                                if file.tree
+                                else ""
+                            ),
+                            **file.time_dict,
+                        }
+                    )
+
     def as_records(self) -> List[dict]:
         """Return the result as a list of dictionaries.
 
-        Each record contains a key specifying the filepath, and a list of violations. This
-        method is useful for serialization as all objects will be builtin python types
-        (ints, strs).
+        Each record contains a key specifying the filepath, and a list of violations.
+        This method is useful for serialization as all objects will be builtin python
+        types (ints, strs).
         """
         return [
             {
@@ -167,10 +226,46 @@ class LintingResult:
         )
 
     @property
-    def tree(self) -> Optional[BaseSegment]:
+    def tree(self) -> Optional[BaseSegment]:  # pragma: no cover
         """A convenience method for when there is only one file and we want the tree."""
         if len(self.paths) > 1:
             raise ValueError(
-                ".tree() cannot be called when a LintingResult contains more than one path."
+                ".tree() cannot be called when a LintingResult contains more than one "
+                "path."
             )
         return self.paths[0].tree
+
+    TMP_PRS_ERROR_TYPES = (SQLTemplaterError, SQLParseError)
+
+    def count_tmp_prs_errors(self) -> Tuple[int, int]:
+        """Count templating or parse errors before and after filtering."""
+        total_errors = self.num_violations(
+            types=self.TMP_PRS_ERROR_TYPES, filter_ignore=False, filter_warning=False
+        )
+        num_filtered_errors = 0
+        for linted_dir in self.paths:
+            for linted_file in linted_dir.files:
+                num_filtered_errors += linted_file.num_violations(
+                    types=self.TMP_PRS_ERROR_TYPES
+                )
+        return total_errors, num_filtered_errors
+
+    def discard_fixes_for_lint_errors_in_files_with_tmp_or_prs_errors(self) -> None:
+        """Discard lint fixes for files with templating or parse errors."""
+        total_errors = self.num_violations(
+            types=self.TMP_PRS_ERROR_TYPES, filter_ignore=False, filter_warning=False
+        )
+        if total_errors:
+            for linted_dir in self.paths:
+                for linted_file in linted_dir.files:
+                    num_errors = linted_file.num_violations(
+                        types=self.TMP_PRS_ERROR_TYPES,
+                        filter_ignore=False,
+                        filter_warning=False,
+                    )
+                    if num_errors:
+                        # File has errors. Discard all the SQLLintError fixes:
+                        # they are potentially unsafe.
+                        for violation in linted_file.violations:
+                            if isinstance(violation, SQLLintError):
+                                violation.fixes = []
