@@ -7,14 +7,19 @@ Implements various runner types for SQLFluff:
   - Multithread (used only by automated tests)
 """
 from abc import ABC
+import bdb
 import functools
 import logging
+import multiprocessing
 import multiprocessing.dummy
 import signal
 import sys
 import traceback
-from typing import Callable, List
+from typing import Callable, List, Tuple, Iterator
 
+from sqlfluff.core import FluffConfig, Linter
+from sqlfluff.core.errors import SQLFluffSkipFile
+from sqlfluff.core.linter import LintedFile
 
 linter_logger: logging.Logger = logging.getLogger("sqlfluff.linter")
 
@@ -32,12 +37,21 @@ class BaseRunner(ABC):
 
     pass_formatter = True
 
-    def iter_rendered(self, fnames):
+    def iter_rendered(self, fnames: List[str]) -> Iterator[Tuple]:
         """Iterate through rendered files ready for linting."""
-        for fname in fnames:
-            yield fname, self.linter.render_file(fname, self.config)
+        for fname in self.linter.templater.sequence_files(
+            fnames, config=self.config, formatter=self.linter.formatter
+        ):
+            try:
+                yield fname, self.linter.render_file(fname, self.config)
+            except SQLFluffSkipFile as s:
+                linter_logger.warning(str(s))
 
-    def iter_partials(self, fnames, fix: bool = False):
+    def iter_partials(
+        self,
+        fnames: List[str],
+        fix: bool = False,
+    ) -> Iterator[Tuple[str, Callable]]:
         """Iterate through partials for linted files.
 
         Generates filenames and objects which return LintedFiles.
@@ -60,7 +74,7 @@ class BaseRunner(ABC):
 
     def run(self, fnames: List[str], fix: bool):
         """Run linting on the specified list of files."""
-        raise NotImplementedError
+        raise NotImplementedError  # pragma: no cover
 
     @classmethod
     def _init_global(cls, config):
@@ -75,7 +89,7 @@ class BaseRunner(ABC):
     def _handle_lint_path_exception(fname, e):
         if isinstance(e, IOError):
             # IOErrors are caught in commands.py, so propagate it
-            raise (e)
+            raise (e)  # pragma: no cover
         linter_logger.warning(
             f"""Unable to lint {fname} due to an internal error. \
 Please report this as an issue with your query's contents and stacktrace below!
@@ -87,11 +101,13 @@ To hide this warning, add the failing file to .sqlfluffignore
 class SequentialRunner(BaseRunner):
     """Simple runner that does sequential processing."""
 
-    def run(self, fnames, fix):
+    def run(self, fnames: List[str], fix: bool) -> Iterator[LintedFile]:
         """Sequential implementation."""
         for fname, partial in self.iter_partials(fnames, fix=fix):
             try:
                 yield partial()
+            except (bdb.BdbQuit, KeyboardInterrupt):  # pragma: no cover
+                raise
             except Exception as e:
                 self._handle_lint_path_exception(fname, e)
 
@@ -109,7 +125,7 @@ class ParallelRunner(BaseRunner):
         super().__init__(linter, config)
         self.processes = processes
 
-    def run(self, fnames, fix):
+    def run(self, fnames: List[str], fix: bool):
         """Parallel implementation.
 
         Note that the partials are generated one at a time then
@@ -124,7 +140,9 @@ class ParallelRunner(BaseRunner):
         ) as pool:
             try:
                 for lint_result in self._map(
-                    pool, self._apply, self.iter_partials(fnames, fix=fix)
+                    pool,
+                    self._apply,
+                    self.iter_partials(fnames, fix=fix),
                 ):
                     if isinstance(lint_result, DelayedException):
                         try:
@@ -138,7 +156,7 @@ class ParallelRunner(BaseRunner):
                                 lint_result.path, lint_result, only_fixable=fix
                             )
                         yield lint_result
-            except KeyboardInterrupt:
+            except KeyboardInterrupt:  # pragma: no cover
                 # On keyboard interrupt (Ctrl-C), terminate the workers.
                 # Notify the user we've received the signal and are cleaning up,
                 # in case it takes awhile.
@@ -174,7 +192,7 @@ class MultiProcessRunner(ParallelRunner):
     MAP_FUNCTION_NAME = "imap_unordered"
 
     @classmethod
-    def _init_global(cls, config):
+    def _init_global(cls, config):  # pragma: no cover
         super()._init_global(config)
 
         # Disable signal handling in the child processes to let the parent
@@ -210,27 +228,33 @@ class DelayedException(Exception):
 
 
 def get_runner(
-    linter,
-    config,
+    linter: Linter,
+    config: FluffConfig,
     processes: int,
     allow_process_parallelism: bool = True,
-) -> BaseRunner:
-    """Generate a runner instance based on parallel and sytem configuration."""
-    # Python multiprocessing isn't supported in 3.6 and before.
-    # The library exists but we get pickling errors with LintedFile.
-    if processes > 1 and sys.version_info >= (3, 7):
+) -> Tuple[BaseRunner, int]:
+    """Generate a runner instance based on parallel and system configuration.
+
+    The processes argument can be positive or negative.
+    - If positive, the integer is interpreted as the number of processes.
+    - If negative or zero, the integer is interpreted as number_of_cpus - processes.
+
+    e.g.
+    -1 = all cpus but one.
+    0 = all cpus
+    1 = 1 cpu
+
+    """
+    if processes <= 0:
+        processes = max(multiprocessing.cpu_count() + processes, 1)
+
+    if processes > 1:
         # Process parallelism isn't really supported during testing
         # so this flag allows us to fall back to a threaded runner
         # in those cases.
         if allow_process_parallelism:
-            return MultiProcessRunner(linter, config, processes=processes)
+            return MultiProcessRunner(linter, config, processes=processes), processes
         else:
-            return MultiThreadRunner(linter, config, processes=processes)
+            return MultiThreadRunner(linter, config, processes=processes), processes
     else:
-        if processes > 1:
-            linter_logger.warning(
-                "Parallel linting is not supported in Python %s.%s.",
-                sys.version_info.major,
-                sys.version_info.minor,
-            )
-        return SequentialRunner(linter, config)
+        return SequentialRunner(linter, config), processes
