@@ -2,18 +2,18 @@
 
 from typing import List, Optional, Tuple
 
-from sqlfluff.core.parser.helpers import trim_non_code_segments
-from sqlfluff.core.parser.match_result import MatchResult
-from sqlfluff.core.parser.match_wrapper import match_wrapper
-from sqlfluff.core.parser.match_logging import parse_match_logging
 from sqlfluff.core.parser.context import ParseContext
-from sqlfluff.core.parser.segments import BaseSegment, allow_ephemeral
 from sqlfluff.core.parser.grammar.base import (
     BaseGrammar,
     MatchableType,
     cached_method_for_parse_context,
 )
 from sqlfluff.core.parser.grammar.sequence import Sequence, Bracketed
+from sqlfluff.core.parser.helpers import trim_non_code_segments
+from sqlfluff.core.parser.match_logging import parse_match_logging
+from sqlfluff.core.parser.match_result import MatchResult
+from sqlfluff.core.parser.match_wrapper import match_wrapper
+from sqlfluff.core.parser.segments import BaseSegment, allow_ephemeral
 
 
 class AnyNumberOf(BaseGrammar):
@@ -28,13 +28,16 @@ class AnyNumberOf(BaseGrammar):
         super().__init__(*args, **kwargs)
 
     @cached_method_for_parse_context
-    def simple(self, parse_context: ParseContext) -> Optional[List[str]]:
+    def simple(
+        self, parse_context: ParseContext, crumbs: Optional[List[str]] = None
+    ) -> Optional[List[str]]:
         """Does this matcher support a uppercase hash matching route?
 
         AnyNumberOf does provide this, as long as *all* the elements *also* do.
         """
         simple_buff = [
-            opt.simple(parse_context=parse_context) for opt in self._elements
+            opt.simple(parse_context=parse_context, crumbs=crumbs)
+            for opt in self._elements
         ]
         if any(elem is None for elem in simple_buff):
             return None
@@ -53,8 +56,8 @@ class AnyNumberOf(BaseGrammar):
     def _first_non_whitespace(segments) -> Optional[str]:
         """Return the upper first non-whitespace segment in the iterable."""
         for segment in segments:
-            if segment.raw_segments_upper:
-                return segment.raw_segments_upper
+            if segment.first_non_whitespace_segment_raw_upper:
+                return segment.first_non_whitespace_segment_raw_upper
         return None
 
     def _prune_options(
