@@ -5,6 +5,9 @@ from sqlfluff.core.parser.segments.base import (
     BaseFileSegment,
     UnparsableSegment,
     BracketedSegment,
+    IdentitySet,
+    FixPatch,
+    SourceFix,
 )
 from sqlfluff.core.parser.segments.generator import SegmentGenerator
 from sqlfluff.core.parser.segments.raw import (
@@ -23,6 +26,8 @@ from sqlfluff.core.parser.segments.meta import (
     Indent,
     Dedent,
     TemplateSegment,
+    EndOfFile,
+    TemplateLoop,
 )
 
 __all__ = (
@@ -45,4 +50,9 @@ __all__ = (
     "Indent",
     "Dedent",
     "TemplateSegment",
+    "EndOfFile",
+    "TemplateLoop",
+    "IdentitySet",
+    "FixPatch",
+    "SourceFix",
 )
