@@ -4,9 +4,10 @@ This is designed to be the root segment, without
 any children, and the output of the lexer.
 """
 
-from typing import Optional, Tuple
+from typing import List, Optional, Tuple, Set
+from uuid import UUID, uuid4
 
-from sqlfluff.core.parser.segments.base import BaseSegment
+from sqlfluff.core.parser.segments.base import BaseSegment, SourceFix
 from sqlfluff.core.parser.markers import PositionMarker
 
 
@@ -26,9 +27,10 @@ class RawSegment(BaseSegment):
         raw: Optional[str] = None,
         pos_marker: Optional[PositionMarker] = None,
         type: Optional[str] = None,
-        name: Optional[str] = None,
         trim_start: Optional[Tuple[str, ...]] = None,
         trim_chars: Optional[Tuple[str, ...]] = None,
+        source_fixes: Optional[List[SourceFix]] = None,
+        uuid: Optional[UUID] = None,
     ):
         """Initialise raw segment.
 
@@ -47,18 +49,25 @@ class RawSegment(BaseSegment):
         self.pos_marker: PositionMarker = pos_marker  # type: ignore
         # if a surrogate type is provided, store it for later.
         self._surrogate_type = type
-        self._surrogate_name = name
         # What should we trim off the ends to get to content
         self.trim_start = trim_start
         self.trim_chars = trim_chars
         # A cache variable for expandable
         self._is_expandable = None
+        # Keep track of any source fixes
+        self._source_fixes = source_fixes
+        # UUID for matching
+        self.uuid = uuid or uuid4()
 
     def __repr__(self):
         return "<{}: ({}) {!r}>".format(
             self.__class__.__name__, self.pos_marker, self.raw
         )
 
+    def __setattr__(self, key, value):
+        """Overwrite BaseSegment's __setattr__ with BaseSegment's superclass."""
+        super(BaseSegment, self).__setattr__(key, value)
+
     # ################ PUBLIC PROPERTIES
 
     @property
@@ -86,11 +95,21 @@ class RawSegment(BaseSegment):
         """Return True if this segment is whitespace."""
         return self._is_whitespace
 
+    @property
+    def raw(self):
+        """Returns the raw segment."""
+        return self._raw
+
     @property
     def raw_upper(self):
-        """Make an uppercase string from the segments of this segment."""
+        """Returns the raw segment in uppercase."""
         return self._raw_upper
 
+    @property
+    def raw_segments(self):
+        """Returns self to be compatible with calls to its superclass."""
+        return [self]
+
     @property
     def segments(self):
         """Return an empty list of child segments.
@@ -99,8 +118,27 @@ class RawSegment(BaseSegment):
         """
         return []
 
+    @property
+    def class_types(self) -> Set[str]:
+        """The set of full types for this segment, including inherited.
+
+        Add the surrogate type for raw segments.
+        """
+        return (
+            {self._surrogate_type} if self._surrogate_type else set()
+        ) | super().class_types
+
+    @property
+    def source_fixes(self) -> List[SourceFix]:
+        """Return any source fixes as list."""
+        return self._source_fixes or []
+
     # ################ INSTANCE METHODS
 
+    def invalidate_caches(self):
+        """Overwrite superclass functionality."""
+        pass
+
     def get_type(self):
         """Returns the type of this segment as a string."""
         return self._surrogate_type or self.type
@@ -111,9 +149,9 @@ class RawSegment(BaseSegment):
             return True
         return self.class_is_type(*seg_type)
 
-    def iter_raw_seg(self):
+    def get_raw_segments(self):
         """Iterate raw segments, mostly for searching."""
-        yield self
+        return [self]
 
     def raw_trimmed(self):
         """Return a trimmed version of the raw content."""
@@ -135,14 +173,10 @@ class RawSegment(BaseSegment):
             return raw_buff
         return raw_buff
 
-    def raw_list(self):
+    def raw_list(self):  # pragma: no cover TODO?
         """Return a list of the raw content of this segment."""
         return [self.raw]
 
-    def _reconstruct(self):
-        """Return a string of the raw content of this segment."""
-        return self._raw
-
     def stringify(self, ident=0, tabsize=4, code_only=False):
         """Use indentation to render this segment and its children as a string."""
         preface = self._preface(ident=ident, tabsize=tabsize)
@@ -155,7 +189,9 @@ class RawSegment(BaseSegment):
         """
         return f"{self.raw!r}"
 
-    def edit(self, raw):
+    def edit(
+        self, raw: Optional[str] = None, source_fixes: Optional[List[SourceFix]] = None
+    ):
         """Create a new segment, with exactly the same position but different content.
 
         Returns:
@@ -163,14 +199,16 @@ class RawSegment(BaseSegment):
 
         Used mostly by fixes.
 
+        NOTE: This *doesn't* copy the uuid. The edited segment is a new segment.
+
         """
         return self.__class__(
-            raw=raw,
+            raw=raw or self.raw,
             pos_marker=self.pos_marker,
             type=self._surrogate_type,
-            name=self._surrogate_name,
             trim_start=self.trim_start,
             trim_chars=self.trim_chars,
+            source_fixes=source_fixes or self.source_fixes,
         )
 
 
@@ -186,7 +224,7 @@ class CodeSegment(RawSegment):
 class UnlexableSegment(CodeSegment):
     """A placeholder to unlexable sections.
 
-    This otherwise behaves exaclty like a code section.
+    This otherwise behaves exactly like a code section.
     """
 
     type = "unlexable"
@@ -196,7 +234,6 @@ class CommentSegment(RawSegment):
     """Segment containing a comment."""
 
     type = "comment"
-    _name = "comment"
     _is_code = False
     _is_comment = True
 
@@ -205,7 +242,6 @@ class WhitespaceSegment(RawSegment):
     """Segment containing whitespace."""
 
     type = "whitespace"
-    _name = "whitespace"
     _is_whitespace = True
     _is_code = False
     _is_comment = False
@@ -223,7 +259,6 @@ class NewlineSegment(RawSegment):
     """
 
     type = "newline"
-    _name = "newline"
     _is_whitespace = True
     _is_code = False
     _is_comment = False
@@ -245,15 +280,17 @@ class KeywordSegment(CodeSegment):
         raw: Optional[str] = None,
         pos_marker: Optional[PositionMarker] = None,
         type: Optional[str] = None,
-        name: Optional[str] = None,
+        source_fixes: Optional[List[SourceFix]] = None,
     ):
         """If no other name is provided we extrapolate it from the raw."""
-        if raw and not name:
-            # names are all lowercase by convention.
-            name = raw.lower()
-        super().__init__(raw=raw, pos_marker=pos_marker, type=type, name=name)
+        super().__init__(
+            raw=raw,
+            pos_marker=pos_marker,
+            type=type,
+            source_fixes=source_fixes,
+        )
 
-    def edit(self, raw):
+    def edit(self, raw=None, source_fixes=None):
         """Create a new segment, with exactly the same position but different content.
 
         Returns:
@@ -261,12 +298,14 @@ class KeywordSegment(CodeSegment):
 
         Used mostly by fixes.
 
+        NOTE: This *doesn't* copy the uuid. The edited segment is a new segment.
+
         """
         return self.__class__(
-            raw=raw,
+            raw=raw or self.raw,
             pos_marker=self.pos_marker,
             type=self._surrogate_type,
-            name=self._surrogate_name,
+            source_fixes=source_fixes or self.source_fixes,
         )
 
 
