@@ -27,9 +27,12 @@ STANDARD_CONFIG_INFO_DICT = {
         "validation": ["space", "tab"],
         "definition": "Whether to use tabs or spaces to add new indents.",
     },
-    "comma_style": {
-        "validation": ["leading", "trailing"],
-        "definition": "The comma style to enforce.",
+    "hanging_indents": {
+        "validation": [True, False],
+        "definition": (
+            "Whether hanging indents will be considered when evaluating the "
+            "indentation of a file."
+        ),
     },
     "allow_scalar": {
         "validation": [True, False],
@@ -91,6 +94,15 @@ STANDARD_CONFIG_INFO_DICT = {
     "ignore_words": {
         "definition": ("Comma separated list of words to ignore from rule"),
     },
+    "ignore_words_regex": {
+        "definition": (
+            "Words to ignore from rule if they are a partial match for the regular "
+            "expression. To ignore only full matches you can use ``^`` (beginning "
+            "of text) and ``$`` (end of text). Due to regular expression operator "
+            "precedence, it is good practice to use parentheses around everything "
+            "between ``^`` and ``$``."
+        ),
+    },
     "forbid_subquery_in": {
         "validation": ["join", "from", "both"],
         "definition": "Which clauses should be linted for subqueries?",
@@ -110,9 +122,13 @@ STANDARD_CONFIG_INFO_DICT = {
     "aliasing": {
         "validation": ["implicit", "explicit"],
         "definition": (
-            "Should alias have an explict AS or is implicit aliasing required?"
+            "Should alias have an explicit AS or is implicit aliasing required?"
         ),
     },
+    "fully_qualify_join_types": {
+        "validation": ["inner", "outer", "both"],
+        "definition": ("Which types of JOIN clauses should be fully qualified?"),
+    },
     "multiline_newline": {
         "validation": [True, False],
         "definition": (
@@ -157,6 +173,39 @@ STANDARD_CONFIG_INFO_DICT = {
             "in statements."
         ),
     },
+    "blocked_regex": {
+        "definition": (
+            "Optional, regex of blocked pattern which should not be used in statements."
+        ),
+    },
+    "preferred_quoted_literal_style": {
+        "validation": ["consistent", "single_quotes", "double_quotes"],
+        "definition": (
+            "Preferred quoting style to use for the quoted literals. If set to "
+            "``consistent`` quoting style is derived from the first quoted literal"
+            "in the file."
+        ),
+    },
+    "min_alias_length": {
+        "validation": range(1000),
+        "definition": (
+            "The minimum length of an alias to allow without raising a violation."
+        ),
+    },
+    "max_alias_length": {
+        "validation": range(1000),
+        "definition": (
+            "The maximum length of an alias to allow without raising a violation."
+        ),
+    },
+    "wildcard_policy": {
+        "validation": ["single", "multiple"],
+        "definition": "Treatment of wildcards. Defaults to ``single``.",
+    },
+    "preferred_type_casting_style": {
+        "validation": ["consistent", "shorthand", "convert", "cast"],
+        "definition": ("The expectation for using sql type casting"),
+    },
 }
 
 
