@@ -4,7 +4,7 @@ from sqlfluff.core.templaters.base import TemplatedFile
 
 # Although these shouldn't usually be instantiated from here
 # we import them to make sure they get registered.
-from sqlfluff.core.templaters.base import RawTemplater
+from sqlfluff.core.templaters.base import RawTemplater, RawFileSlice
 from sqlfluff.core.templaters.jinja import JinjaTemplater
 from sqlfluff.core.templaters.python import PythonTemplater
 from sqlfluff.core.templaters.placeholder import PlaceholderTemplater
@@ -16,6 +16,7 @@ def core_templaters():
 
 
 __all__ = (
+    "RawFileSlice",
     "TemplatedFile",
     "RawTemplater",
     "JinjaTemplater",
