@@ -469,7 +469,7 @@ postgres_docs_keywords = [
     ("MAP", "not-keyword"),
     ("MAPPING", "non-reserved"),
     ("MATCH", "non-reserved"),
-    ("MATCHED", "not-keyword"),
+    ("MATCHED", "non-reserved"),
     ("MATCHES", "not-keyword"),
     ("MATCH_NUMBER", "not-keyword"),
     ("MATCH_RECOGNIZE", "not-keyword"),
@@ -478,7 +478,7 @@ postgres_docs_keywords = [
     ("MAXVALUE", "non-reserved"),
     ("MEASURES", "not-keyword"),
     ("MEMBER", "not-keyword"),
-    ("MERGE", "not-keyword"),
+    ("MERGE", "non-reserved"),
     ("MESSAGE_LENGTH", "not-keyword"),
     ("MESSAGE_OCTET_LENGTH", "not-keyword"),
     ("MESSAGE_TEXT", "not-keyword"),
@@ -916,12 +916,16 @@ postgres_nondocs_keywords = [
     ("ALLOW_CONNECTIONS", "non-reserved"),
     ("BREADTH", "non-reserved"),
     ("BUFFERS", "non-reserved"),
+    ("BYPASSRLS", "non-reserved"),
     ("CONNECT", "reserved"),
     ("COSTS", "non-reserved"),
     ("CURRENT_USER", "non-reserved"),
+    ("CREATEDB", "non-reserved"),
+    ("CREATEROLE", "non-reserved"),
     ("DATE", "non-reserved"),
     ("DEPTH", "non-reserved"),
     ("DESCRIBE", "non-reserved"),
+    ("DETERMINISTIC", "non-reserved"),
     ("EXTENDED", "non-reserved"),
     ("FILE", "non-reserved"),
     ("FORCE_NOT_NULL", "non-reserved"),
@@ -929,29 +933,42 @@ postgres_nondocs_keywords = [
     ("FORCE_QUOTE", "non-reserved"),
     ("FORMAT", "non-reserved"),
     ("HASH", "non-reserved"),
+    ("ICU", "non-reserved"),
     ("IGNORE", "non-reserved"),
     ("IS_TEMPLATE", "non-reserved"),
     ("JSON", "non-reserved"),
     ("LC_COLLATE", "non-reserved"),
     ("LC_CTYPE", "non-reserved"),
+    ("LIBC", "non-reserved"),
     ("LIST", "non-reserved"),
+    ("LOGIN", "non-reserved"),
     ("LOCALE", "non-reserved"),
     ("MAIN", "non-reserved"),
     ("MODULUS", "non-reserved"),
+    ("NOBYPASSRLS", "non-reserved"),
+    ("NOCREATEDB", "non-reserved"),
+    ("NOCREATEROLE", "non-reserved"),
+    ("NOINHERIT", "non-reserved"),
+    ("NOLOGIN", "non-reserved"),
+    ("NOREPLICATION", "non-reserved"),
+    ("NOSUPERUSER", "non-reserved"),
     ("PLAIN", "non-reserved"),
+    ("PROVIDER", "non-reserved"),
     ("PUBLIC", "non-reserved"),
     ("REMAINDER", "non-reserved"),
+    ("REPLICATION", "non-reserved"),
     ("RESPECT", "non-reserved"),
     ("RESTRICTED", "non-reserved"),
     ("SAFE", "non-reserved"),
     ("SETTINGS", "non-reserved"),
     ("SKIP_LOCKED", "non-reserved"),
     ("SUMMARY", "non-reserved"),
+    ("SUPERUSER", "non-reserverd"),
     ("TIMETZ", "non-reserved"),
     ("TIMESTAMPTZ", "non-reserved"),
     ("TIMING", "non-reserved"),
     ("UNSAFE", "non-reserved"),
-    ("USAGE", "reserved"),
+    ("USAGE", "non-reserved"),
     ("WAL", "non-reserved"),
 ]
 
