@@ -9,23 +9,25 @@ Teradata Database SQL Data Definition Language Syntax and Examples
 """
 
 from sqlfluff.core.parser import (
+    AnyNumberOf,
+    Anything,
     BaseSegment,
+    Bracketed,
+    CodeSegment,
+    Dedent,
+    Delimited,
+    Indent,
+    Matchable,
+    OneOf,
+    OptionallyBracketed,
+    Ref,
+    RegexLexer,
     Sequence,
     StartsWith,
-    OneOf,
-    Delimited,
-    Bracketed,
-    AnyNumberOf,
-    Ref,
-    Anything,
-    RegexLexer,
-    CodeSegment,
-    Indent,
-    Dedent,
-    OptionallyBracketed,
 )
 
 from sqlfluff.core.dialects import load_raw_dialect
+from sqlfluff.dialects import dialect_ansi as ansi
 
 ansi_dialect = load_raw_dialect("ansi")
 teradata_dialect = ansi_dialect.copy_as("teradata")
@@ -33,7 +35,12 @@ teradata_dialect = ansi_dialect.copy_as("teradata")
 teradata_dialect.patch_lexer_matchers(
     [
         # so it also matches 1.
-        RegexLexer("numeric_literal", r"([0-9]+(\.[0-9]*)?)", CodeSegment),
+        RegexLexer(
+            "numeric_literal",
+            r"([0-9]+(\.[0-9]*)?)",
+            CodeSegment,
+            segment_kwargs={"type": "numeric_literal"},
+        ),
     ]
 )
 
@@ -77,6 +84,7 @@ teradata_dialect.sets("unreserved_keywords").update(
         "RUN",
         "SAMPLE",
         "SEL",
+        "SS",
         "STAT",
         "SUMMARY",
         "THRESHOLD",
@@ -91,7 +99,6 @@ teradata_dialect.sets("bare_functions").update(["DATE"])
 
 
 # BTEQ statement
-@teradata_dialect.segment()
 class BteqKeyWordSegment(BaseSegment):
     """Bteq Keywords.
 
@@ -132,7 +139,6 @@ class BteqKeyWordSegment(BaseSegment):
     )
 
 
-@teradata_dialect.segment()
 class BteqStatementSegment(BaseSegment):
     """Bteq statements start with a dot, followed by a Keyword.
 
@@ -144,8 +150,7 @@ class BteqStatementSegment(BaseSegment):
     """
 
     type = "bteq_statement"
-    match_grammar = StartsWith(Ref("DotSegment"))
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         Ref("DotSegment"),
         Ref("BteqKeyWordSegment"),
         AnyNumberOf(
@@ -159,7 +164,6 @@ class BteqStatementSegment(BaseSegment):
     )
 
 
-@teradata_dialect.segment()
 class TdCollectStatUsingOptionClauseSegment(BaseSegment):
     """'using_option' for COLLECT STAT clause."""
 
@@ -187,7 +191,6 @@ class TdCollectStatUsingOptionClauseSegment(BaseSegment):
     )
 
 
-@teradata_dialect.segment()
 class TdOrderByStatClauseSegment(BaseSegment):
     """An `ORDER BY (VALUES|HASH) (column_name)` clause in COLLECT STATS."""
 
@@ -198,7 +201,6 @@ class TdOrderByStatClauseSegment(BaseSegment):
 
 
 # Collect Statistics statement
-@teradata_dialect.segment()
 class TdCollectStatisticsStatementSegment(BaseSegment):
     """A `COLLECT STATISTICS (Optimizer Form)` statement.
 
@@ -229,11 +231,7 @@ class TdCollectStatisticsStatementSegment(BaseSegment):
                     "INDEX",
                     Ref("IndexReferenceSegment", optional=True),
                     Ref.keyword("ALL", optional=True),
-                    Bracketed(
-                        Delimited(
-                            Ref("ColumnReferenceSegment"), delimiter=Ref("CommaSegment")
-                        )
-                    ),
+                    Bracketed(Delimited(Ref("ColumnReferenceSegment"))),
                     Ref("TdOrderByStatClauseSegment", optional=True),
                 ),
                 # UNIQUE INDEX index_name
@@ -252,7 +250,6 @@ class TdCollectStatisticsStatementSegment(BaseSegment):
                                 Ref.keyword("PARTITION"),
                                 # TODO: expression
                             ),
-                            delimiter=Ref("CommaSegment"),
                         ),
                     ),
                     Sequence(
@@ -262,7 +259,6 @@ class TdCollectStatisticsStatementSegment(BaseSegment):
                     ),
                 ),
             ),
-            delimiter=Ref("CommaSegment"),
             optional=True,
         ),
         "ON",
@@ -271,7 +267,6 @@ class TdCollectStatisticsStatementSegment(BaseSegment):
     )
 
 
-@teradata_dialect.segment()
 class TdCommentStatementSegment(BaseSegment):
     """A `COMMENT` statement.
 
@@ -316,7 +311,6 @@ class TdCommentStatementSegment(BaseSegment):
 
 
 # Rename table statement
-@teradata_dialect.segment()
 class TdRenameStatementSegment(BaseSegment):
     """A `RENAME TABLE` statement.
 
@@ -338,19 +332,17 @@ class TdRenameStatementSegment(BaseSegment):
 
 
 # Adding Teradata specific DATE FORMAT 'YYYYMM'
-@teradata_dialect.segment(replace=True)
-class DatatypeSegment(BaseSegment):
+class DatatypeSegment(ansi.DatatypeSegment):
     """A data type segment.
 
     DATE FORMAT 'YYYY-MM-DD'
     """
 
-    type = "td_internal_data_type"
     match_grammar = Sequence(
         Ref("DatatypeIdentifierSegment"),
         Bracketed(
             OneOf(
-                Delimited(Ref("ExpressionSegment"), delimiter=Ref("CommaSegment")),
+                Delimited(Ref("ExpressionSegment")),
                 # The brackets might be empty for some cases...
                 optional=True,
             ),
@@ -363,7 +355,6 @@ class DatatypeSegment(BaseSegment):
     )
 
 
-@teradata_dialect.segment()
 class TeradataCastSegment(BaseSegment):
     """A casting operation using Teradata conversion syntax.
 
@@ -387,7 +378,6 @@ class TeradataCastSegment(BaseSegment):
     match_grammar = Bracketed(Ref("DatatypeSegment"))
 
 
-@teradata_dialect.segment(replace=True)
 class ExpressionSegment(BaseSegment):
     """A expression, either arithmetic or boolean.
 
@@ -403,7 +393,6 @@ class ExpressionSegment(BaseSegment):
 
 
 # Adding Teradata specific column definitions
-@teradata_dialect.segment(replace=True)
 class ColumnDefinitionSegment(BaseSegment):
     """A column definition, e.g. for CREATE TABLE or ALTER TABLE."""
 
@@ -420,7 +409,6 @@ class ColumnDefinitionSegment(BaseSegment):
     )
 
 
-@teradata_dialect.segment()
 class TdColumnConstraintSegment(BaseSegment):
     """Teradata specific column attributes.
 
@@ -441,9 +429,7 @@ class TdColumnConstraintSegment(BaseSegment):
             Sequence(  # COMPRESS [(1.,3.) | 3. | NULL],
                 "COMPRESS",
                 OneOf(
-                    Bracketed(
-                        Delimited(Ref("LiteralGrammar"), delimiter=Ref("CommaSegment"))
-                    ),
+                    Bracketed(Delimited(Ref("LiteralGrammar"))),
                     Ref("LiteralGrammar"),
                     "NULL",
                     optional=True,
@@ -454,7 +440,6 @@ class TdColumnConstraintSegment(BaseSegment):
 
 
 # Create Teradata Create Table Statement
-@teradata_dialect.segment()
 class TdCreateTableOptions(BaseSegment):
     """CreateTableOptions.
 
@@ -463,9 +448,9 @@ class TdCreateTableOptions(BaseSegment):
     """
 
     type = "create_table_options_statement"
-    match_grammar = AnyNumberOf(
-        Sequence(
-            Ref("CommaSegment"),
+    match_grammar = Sequence(
+        Ref("CommaSegment"),
+        Delimited(
             OneOf(
                 # [ NO ] FALLBACK [ PROTECTION ]
                 Sequence(
@@ -511,7 +496,6 @@ class TdCreateTableOptions(BaseSegment):
     )
 
 
-@teradata_dialect.segment()
 class TdTablePartitioningLevel(BaseSegment):
     """Partitioning Level.
 
@@ -538,13 +522,11 @@ class TdTablePartitioningLevel(BaseSegment):
                     Ref("FunctionNameSegment"),
                     Bracketed(Anything(optional=True)),
                 ),
-                delimiter=Ref("CommaSegment"),
             ),
         ),
     )
 
 
-@teradata_dialect.segment()
 class TdTableConstraints(BaseSegment):
     """Teradata specific table attributes.
 
@@ -555,50 +537,48 @@ class TdTableConstraints(BaseSegment):
     """
 
     type = "td_table_constraint"
-    match_grammar = Sequence(
-        AnyNumberOf(
-            # PRIMARY Index
-            OneOf(
-                Sequence(  # UNIQUE PRIMARY INDEX Column_name | ( Column_name, ... )
-                    Ref.keyword("UNIQUE", optional=True),
-                    "PRIMARY",
-                    "INDEX",
-                    Ref("ObjectReferenceSegment", optional=True),  # primary index name
-                    OneOf(
-                        Bracketed(
-                            Delimited(
-                                Ref("SingleIdentifierGrammar"),
-                                delimiter=Ref("CommaSegment"),
-                            )
-                        ),
-                        Ref("SingleIdentifierGrammar"),
-                    ),
-                ),
-                Sequence("NO", "PRIMARY", "INDEX"),  # NO PRIMARY INDEX
-            ),
-            # PARTITION BY ...
-            Sequence(  # INDEX HOPR_TRN_TRAV_SIN_MP_I ( IND_TIPO_TARJETA );
-                "PARTITION",
-                "BY",
-                Ref("TdTablePartitioningLevel"),
-            ),
-            # Index
-            Sequence(  # INDEX HOPR_TRN_TRAV_SIN_MP_I ( IND_TIPO_TARJETA );
+    match_grammar = AnyNumberOf(
+        # PRIMARY Index
+        OneOf(
+            Sequence(  # UNIQUE PRIMARY INDEX Column_name | ( Column_name, ... )
                 Ref.keyword("UNIQUE", optional=True),
+                "PRIMARY",
                 "INDEX",
-                Ref("ObjectReferenceSegment"),  # Index name
-                Ref.keyword("ALL", optional=True),
-                Bracketed(  # Columns making up  constraint
-                    Delimited(
-                        Ref("ColumnReferenceSegment"), delimiter=Ref("CommaSegment")
+                Ref("ObjectReferenceSegment", optional=True),  # primary index name
+                OneOf(
+                    Bracketed(
+                        Delimited(
+                            Ref("SingleIdentifierGrammar"),
+                        )
                     ),
+                    Ref("SingleIdentifierGrammar"),
                 ),
             ),
-        )
+            Sequence("NO", "PRIMARY", "INDEX"),  # NO PRIMARY INDEX
+        ),
+        # PARTITION BY ...
+        Sequence(  # INDEX HOPR_TRN_TRAV_SIN_MP_I ( IND_TIPO_TARJETA );
+            "PARTITION",
+            "BY",
+            Ref("TdTablePartitioningLevel"),
+        ),
+        # Index
+        Sequence(  # INDEX HOPR_TRN_TRAV_SIN_MP_I ( IND_TIPO_TARJETA );
+            Ref.keyword("UNIQUE", optional=True),
+            "INDEX",
+            Ref("ObjectReferenceSegment"),  # Index name
+            Ref.keyword("ALL", optional=True),
+            Bracketed(  # Columns making up  constraint
+                Delimited(Ref("ColumnReferenceSegment")),
+            ),
+        ),
+        # WITH DATA
+        Sequence("WITH", Sequence("NO", optional=True), "DATA"),
+        # ON COMMIT PRESERVE ROWS
+        Sequence("ON", "COMMIT", OneOf("PRESERVE", "DELETE"), "ROWS"),
     )
 
 
-@teradata_dialect.segment(replace=True)
 class CreateTableStatementSegment(BaseSegment):
     """A `CREATE [MULTISET| SET] TABLE` statement."""
 
@@ -613,7 +593,7 @@ class CreateTableStatementSegment(BaseSegment):
         Sequence("IF", "NOT", "EXISTS", optional=True),
         Ref("TableReferenceSegment"),
         # , NO FALLBACK, NO BEFORE JOURNAL, NO AFTER JOURNAL
-        OneOf(Ref("TdCreateTableOptions"), optional=True),
+        Ref("TdCreateTableOptions", optional=True),
         OneOf(
             # Columns and comment syntax:
             Sequence(
@@ -623,7 +603,6 @@ class CreateTableStatementSegment(BaseSegment):
                             Ref("ColumnDefinitionSegment"),
                             Ref("TableConstraintSegment"),
                         ),
-                        delimiter=Ref("CommaSegment"),
                     )
                 ),
                 Ref("CommentClauseSegment", optional=True),
@@ -639,7 +618,6 @@ class CreateTableStatementSegment(BaseSegment):
 
 
 # Update
-@teradata_dialect.segment(replace=True)
 class UpdateStatementSegment(BaseSegment):
     """A `Update from` statement.
 
@@ -651,8 +629,7 @@ class UpdateStatementSegment(BaseSegment):
     """
 
     type = "update_statement"
-    match_grammar = StartsWith("UPDATE")
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         "UPDATE",
         OneOf(
             Ref("TableReferenceSegment"),
@@ -667,7 +644,6 @@ class UpdateStatementSegment(BaseSegment):
     )
 
 
-@teradata_dialect.segment()
 class FromUpdateClauseSegment(BaseSegment):
     """A `FROM` clause like in `SELECT` but terminated by SET."""
 
@@ -678,33 +654,33 @@ class FromUpdateClauseSegment(BaseSegment):
         Delimited(
             # Optional old school delimited joins
             Ref("FromExpressionElementSegment"),
-            delimiter=Ref("CommaSegment"),
         ),
     )
 
 
 # Adding Teradata specific statements
-@teradata_dialect.segment(replace=True)
-class StatementSegment(BaseSegment):
+class StatementSegment(ansi.StatementSegment):
     """A generic segment, to any of its child subsegments."""
 
     type = "statement"
 
-    parse_grammar = ansi_dialect.get_segment("StatementSegment").parse_grammar.copy(
+    parse_grammar = ansi.StatementSegment.parse_grammar.copy(
         insert=[
             Ref("TdCollectStatisticsStatementSegment"),
             Ref("BteqStatementSegment"),
             Ref("TdRenameStatementSegment"),
             Ref("QualifyClauseSegment"),
             Ref("TdCommentStatementSegment"),
+            Ref("DatabaseStatementSegment"),
+            Ref("SetSessionStatementSegment"),
         ],
     )
 
-    match_grammar = ansi_dialect.get_segment("StatementSegment").match_grammar.copy()
+    match_grammar = ansi.StatementSegment.match_grammar.copy()
 
 
 teradata_dialect.add(
-    TdCastIdentifierSegment=Sequence(
+    TdCastIdentifierGrammar=Sequence(
         OneOf("DATE", "TIMESTAMP"), Ref("ExpressionSegment")
     ),
 )
@@ -713,7 +689,7 @@ teradata_dialect.replace(
     SingleIdentifierGrammar=OneOf(
         Ref("NakedIdentifierSegment"),
         Ref("QuotedIdentifierSegment"),
-        Ref("TdCastIdentifierSegment"),
+        Ref("TdCastIdentifierGrammar"),
     ),
     SelectClauseSegmentGrammar=Sequence(
         OneOf("SELECT", "SEL"),
@@ -727,7 +703,6 @@ teradata_dialect.replace(
 )
 
 
-@teradata_dialect.segment()
 class QualifyClauseSegment(BaseSegment):
     """A `QUALIFY` clause like in `SELECT`."""
 
@@ -745,47 +720,32 @@ class QualifyClauseSegment(BaseSegment):
     )
 
 
-@teradata_dialect.segment(replace=True)
-class SelectStatementSegment(BaseSegment):
+class SelectStatementSegment(ansi.SelectStatementSegment):
     """A `SELECT` statement.
 
     https://dev.mysql.com/doc/refman/5.7/en/select.html
     """
 
-    type = "select_statement"
-    match_grammar = ansi_dialect.get_segment(
-        "SelectStatementSegment"
-    ).match_grammar.copy()
-
-    parse_grammar = ansi_dialect.get_segment(
-        "SelectStatementSegment"
-    ).parse_grammar.copy(
+    match_grammar = ansi.SelectStatementSegment.match_grammar
+    parse_grammar = ansi.SelectStatementSegment.parse_grammar.copy(
         insert=[Ref("QualifyClauseSegment", optional=True)],
         before=Ref("OrderByClauseSegment", optional=True),
     )
 
 
-@teradata_dialect.segment(replace=True)
-class UnorderedSelectStatementSegment(BaseSegment):
+class UnorderedSelectStatementSegment(ansi.UnorderedSelectStatementSegment):
     """An unordered `SELECT` statement.
 
     https://dev.mysql.com/doc/refman/5.7/en/select.html
     """
 
-    type = "select_statement"
-    match_grammar = ansi_dialect.get_segment(
-        "UnorderedSelectStatementSegment"
-    ).match_grammar.copy()
-
-    parse_grammar = ansi_dialect.get_segment(
-        "UnorderedSelectStatementSegment"
-    ).parse_grammar.copy(
+    match_grammar = ansi.UnorderedSelectStatementSegment.match_grammar
+    parse_grammar = ansi.UnorderedSelectStatementSegment.parse_grammar.copy(
         insert=[Ref("QualifyClauseSegment", optional=True)],
         before=Ref("OverlapsClauseSegment", optional=True),
     )
 
 
-@teradata_dialect.segment(replace=True)
 class SelectClauseModifierSegment(BaseSegment):
     """Things that come after SELECT but before the columns.
 
@@ -822,14 +782,12 @@ class SelectClauseModifierSegment(BaseSegment):
     )
 
 
-@teradata_dialect.segment(replace=True)
-class SelectClauseSegment(BaseSegment):
+class SelectClauseSegment(ansi.SelectClauseSegment):
     """A group of elements in a select target statement.
 
     Remove OVERLAPS as a terminator as this can be part of SelectClauseModifierSegment
     """
 
-    type = "select_clause"
     match_grammar = StartsWith(
         Sequence(
             OneOf("SELECT", "SEL"), Ref("WildcardExpressionSegment", optional=True)
@@ -843,5 +801,35 @@ class SelectClauseSegment(BaseSegment):
         ),
         enforce_whitespace_preceding_terminator=True,
     )
+    parse_grammar = ansi.SelectClauseSegment.parse_grammar
 
-    parse_grammar = ansi_dialect.get_segment("SelectClauseSegment").parse_grammar.copy()
+
+class DatabaseStatementSegment(BaseSegment):
+    """A `DATABASE` statement.
+
+    https://docs.teradata.com/r/Teradata-Database-SQL-Data-Definition-Language-Syntax-and-Examples/December-2015/Database-Statements/DATABASE
+    """
+
+    type = "database_statement"
+    match_grammar: Matchable = Sequence(
+        "DATABASE",
+        Ref("DatabaseReferenceSegment"),
+    )
+
+
+# Limited to SET SESSION DATABASE for now.
+# Many other session parameters may be set via SET SESSION.
+class SetSessionStatementSegment(BaseSegment):
+    """A `SET SESSION` statement.
+
+    https://docs.teradata.com/r/Teradata-Database-SQL-Data-Definition-Language-Syntax-and-Examples/December-2015/Session-Statements/SET-SESSION-DATABASE
+    """
+
+    type = "set_session_statement"
+    match_grammar: Matchable = Sequence(
+        OneOf(
+            Sequence("SET", "SESSION"),
+            "SS",
+        ),
+        Ref("DatabaseStatementSegment"),
+    )
