@@ -9,6 +9,7 @@ RESERVED_KEYWORDS = [
     "ALTER",
     "AND",
     "ANY",
+    "APPEND",
     "AS",
     "ASC",
     "AUTHORIZATION",
@@ -55,6 +56,7 @@ RESERVED_KEYWORDS = [
     "DISTRIBUTED",
     "DOUBLE",
     "DROP",
+    "DYNAMIC",
     "ELSE",
     "END",
     "ERRLVL",
@@ -65,16 +67,21 @@ RESERVED_KEYWORDS = [
     "EXISTS",
     "EXIT",
     "EXTERNAL",
+    "FAST_FORWARD",
     "FETCH",
     "FILE",
     "FILLFACTOR",
     "FOR",
+    "FORWARD_ONLY",
     "FOREIGN",
     "FREETEXT",
     "FREETEXTTABLE",
     "FROM",
     "FULL",
+    "FULLSCAN",
     "FUNCTION",
+    "GLOBAL",
+    "GO",
     "GOTO",
     "GRANT",
     "GROUP",
@@ -93,12 +100,15 @@ RESERVED_KEYWORDS = [
     "IS",
     "JOIN",
     "KEY",
+    "KEYSET",
     "KILL",
     "LEFT",
     "LIKE",
     "LINENO",
+    "LOCAL",
     "MERGE",
     "NATIONAL",
+    "NATIVE_COMPILATION",
     "NOCHECK",
     "NONCLUSTERED",
     "NOT",
@@ -113,6 +123,7 @@ RESERVED_KEYWORDS = [
     "OPENQUERY",
     "OPENROWSET",
     "OPENXML",
+    "OPTIMISTIC",
     "OPTION",
     "OR",
     "ORDER",
@@ -128,10 +139,12 @@ RESERVED_KEYWORDS = [
     "PUBLIC",
     "RAISERROR",
     "READ",
+    "READ_ONLY",
     "READTEXT",
     "RECONFIGURE",
     "REFERENCES",
     "REPLICATION",
+    "RESAMPLE",
     "RESTORE",
     "RESTRICT",
     "RETURN",
@@ -144,6 +157,8 @@ RESERVED_KEYWORDS = [
     "RULE",
     "SAVE",
     "SCHEMA",
+    "SCROLL",
+    "SCROLL_LOCKS",
     "SELECT",
     "SEMANTICKEYPHRASETABLE",
     "SEMANTICSIMILARITYDETAILSTABLE",
@@ -153,6 +168,7 @@ RESERVED_KEYWORDS = [
     "SETUSER",
     "SHUTDOWN",
     "SOME",
+    "STATIC",
     "STATISTICS",
     "SYSTEM_USER",
     "TABLE",
@@ -168,6 +184,7 @@ RESERVED_KEYWORDS = [
     "TRUNCATE",
     "TRY_CONVERT",
     "TSEQUAL",
+    "TYPE_WARNING",
     "UNION",
     "UNIQUE",
     "UNPIVOT",
@@ -189,6 +206,7 @@ RESERVED_KEYWORDS = [
 
 UNRESERVED_KEYWORDS = [
     "ABORT_AFTER_WAIT",
+    "AFTER",
     "ALGORITHM",
     "ALLOW_PAGE_LOCKS",
     "ALLOW_ROW_LOCKS",
@@ -202,26 +220,35 @@ UNRESERVED_KEYWORDS = [
     "APPLY",
     "ARITHABORT",
     "ARITHIGNORE",
+    "AT",
+    "AUTO",
     "BERNOULLI",
     "BLOCKERS",
+    "BREAK",
     "CACHE",
     "CALLED",
     "CALLER",
     "CAST",
     "CATCH",
+    "CODEPAGE",
     "COLUMN_ENCRYPTION_KEY",
     "COLUMNSTORE",
     "COLUMNSTORE_ARCHIVE",
+    "COMMITTED",
     "CONCAT",
     "CONCAT_NULL_YIELDS_NULL",
+    "CONTINUE",
+    "CONTROL",
     "COMPRESSION_DELAY",
     "CURSOR_CLOSE_ON_COMMIT",
     "CYCLE",
     "DATA_COMPRESSION",
+    "DATASOURCE",
     "DATE",
     "DATEFIRST",
     "DATEFORMAT",
     "DEADLOCK_PRIORITY",
+    "DELAY",
     "DENSE_RANK",
     "DETERMINISTIC",
     "DISABLE",
@@ -232,21 +259,30 @@ UNRESERVED_KEYWORDS = [
     "ENCRYPTED",
     "ENCRYPTION",
     "ENCRYPTION_TYPE",
+    "ERRORFILE",
+    "ERRORFILE_DATA_SOURCE",
     "EXPAND",
     "EXPLAIN",  # Azure Synapse Analytics specific
+    "EXPLICIT",
     "EXTERNALPUSHDOWN",
     "FAST",
+    "FIELDQUOTE",
     "FILESTREAM",
     "FILESTREAM_ON",
     "FILTER",
     "FIPS_FLAGGER",
+    "FIRST",
+    "FIRSTROW",
     "FMTONLY",
     "FOLLOWING",
+    "FOR",
     "FORCE",
     "FORCEPLAN",
     "FORCESCAN",
     "FORCESEEK",
-    "GO",
+    "FORMAT",
+    "FORMATFILE",
+    "FORMATFILE_DATA_SOURCE",
     "HASH",
     "HEAP",  # Azure Synapse Analytics specific
     "HIDDEN",
@@ -260,6 +296,7 @@ UNRESERVED_KEYWORDS = [
     "INCLUDE",
     "INCREMENT",
     "INLINE",
+    "INSTEAD",
     "INTERVAL",
     "IO",
     "ISOLATION",
@@ -268,20 +305,26 @@ UNRESERVED_KEYWORDS = [
     "KEEPDEFAULTS",
     "KEEPFIXED",
     "KEEPIDENTITY",
+    # LABEL is an Azure Synapse Analytics specific reserved keyword
+    # but could break TSQL parsing to add there
     "LABEL",
-    # Azure Synapse Analytics specific, reserved keyword but could break TSQL parsing to
-    # add there
     "LANGUAGE",
+    "LAST",
+    "LASTROW",
     "LEVEL",
     "LOAD",  # listed as reserved but functionally unreserved
     "LOCATION",
     "LOCK_TIMEOUT",
+    "LOG",
+    "LOGIN",
     "LOOP",
     "MAX_DURATION",
     "MAX_GRANT_PERCENT",
     "MASKED",
     "MATCHED",
+    "MAX",
     "MAXDOP",
+    "MAXERRORS",
     "MAXRECURSION",
     "MAXVALUE",
     "MIN_GRANT_PERCENT",
@@ -303,6 +346,7 @@ UNRESERVED_KEYWORDS = [
     "ONLINE",
     "OPTIMIZE",
     "OPTIMIZE_FOR_SEQUENTIAL_KEY",
+    "OUT",
     "OUTPUT",
     "OWNER",
     "PAD_INDEX",
@@ -313,6 +357,7 @@ UNRESERVED_KEYWORDS = [
     "PARSEONLY",
     "PARTITION",
     "PARTITIONS",
+    "PATH",
     "PERCENTILE_CONT",
     "PERCENTILE_DISC",
     "PERSISTED",
@@ -326,10 +371,13 @@ UNRESERVED_KEYWORDS = [
     "RANDOMIZED",
     "RANGE",
     "RANK",
+    "RAW",
     "READCOMMITTED",
     "READCOMMITTEDLOCK",
+    "READONLY",
     "READPAST",
     "READUNCOMMITTED",
+    "RECEIVE",
     "RECOMPILE",
     "RECURSIVE",
     "REMOTE_PROC_TRANSACTIONS",
@@ -355,12 +403,17 @@ UNRESERVED_KEYWORDS = [
     "SCHEMABINDING",
     "SECURITYAUDIT",  # listed as reserved but functionally unreserved
     "SELF",
+    "SETERROR",
     "SEQUENCE",
     "SEQUENCE_NUMBER",
     "SERIALIZABLE",
+    "SERVER",
     "SHOWPLAN_ALL",
     "SHOWPLAN_TEXT",
     "SHOWPLAN_XML",
+    "SINGLE_BLOB",
+    "SINGLE_CLOB",
+    "SINGLE_NCLOB",
     "SNAPSHOT",
     "SORT_IN_TEMPDB",
     "SOURCE",
@@ -370,15 +423,18 @@ UNRESERVED_KEYWORDS = [
     "STATISTICS_INCREMENTAL",
     "STATISTICS_NORECOMPUTE",
     "STRING_AGG",
+    "SYNONYM",
     "SWITCH",
     "SYSTEM",
     "TABLOCK",
     "TABLOCKX",
+    "TAKE",
     "TARGET",
     "TEXTIMAGE_ON",
     "THROW",
     "TIES",
     "TIME",
+    "TIMEOUT",
     "TIMESTAMP",
     "TRANSACTION_ID",
     "TRUNCATE_TARGET",  # Azure Synapse Analytics specific
@@ -386,15 +442,19 @@ UNRESERVED_KEYWORDS = [
     "TYPE",
     "UPDLOCK",
     "UNBOUNDED",
+    "UNCOMMITTED",
     "UNKNOWN",
     "USER_DB",  # Azure Synapse Analytics specific, deprecated
     "USING",
     "VALUE",
     "VIEW_METADATA",
+    "WAITFOR",
     "WAIT_AT_LOW_PRIORITY",
     "WITHIN",
+    "WHILE",
     "WORK",
     "XACT_ABORT",
     "XLOCK",
     "XML",
+    "ZONE",
 ]
