@@ -1,12 +1,14 @@
 """Implementation of Rule L014."""
 
-from typing import Tuple, List
+from typing import Tuple, Optional, List
 
 from sqlfluff.core.parser import BaseSegment
-from sqlfluff.core.rules.base import LintResult, RuleContext
+from sqlfluff.core.rules import LintResult, RuleContext
+from sqlfluff.core.rules.crawlers import SegmentSeekerCrawler
 from sqlfluff.core.rules.doc_decorators import (
     document_configuration,
     document_fix_compatible,
+    document_groups,
 )
 from sqlfluff.rules.L010 import Rule_L010
 
@@ -30,8 +32,9 @@ def identifiers_policy_applicable(
     return False
 
 
-@document_configuration
+@document_groups
 @document_fix_compatible
+@document_configuration
 class Rule_L014(Rule_L010):
     """Inconsistent capitalisation of unquoted identifiers.
 
@@ -67,18 +70,34 @@ class Rule_L014(Rule_L010):
 
     """
 
-    _target_elems: List[Tuple[str, str]] = [("name", "naked_identifier")]
+    groups = ("all", "core")
+    lint_phase = "post"
+    crawl_behaviour = SegmentSeekerCrawler(
+        {"naked_identifier", "properties_naked_identifier"}
+    )
     config_keywords = [
         "extended_capitalisation_policy",
         "unquoted_identifiers_policy",
         "ignore_words",
+        "ignore_words_regex",
     ]
     _description_elem = "Unquoted identifiers"
 
-    def _eval(self, context: RuleContext) -> LintResult:
+    def _eval(self, context: RuleContext) -> Optional[List[LintResult]]:
+        # Return None if identifier is case-sensitive property to enable Change
+        # Data Feed
+        # https://docs.delta.io/2.0.0/delta-change-data-feed.html#enable-change-data-feed
+        if (
+            context.dialect.name in ["sparksql"]
+            and context.parent_stack
+            and context.parent_stack[-1].type == "property_name_identifier"
+            and context.segment.raw == "enableChangeDataFeed"
+        ):
+            return None
+
         if identifiers_policy_applicable(
             self.unquoted_identifiers_policy, context.parent_stack  # type: ignore
         ):
             return super()._eval(context=context)
         else:
-            return LintResult(memory=context.memory)
+            return [LintResult(memory=context.memory)]
