@@ -1,10 +1,12 @@
 """Implementation of Rule L024."""
 
 
-from sqlfluff.core.rules.doc_decorators import document_fix_compatible
+from sqlfluff.core.rules.crawlers import SegmentSeekerCrawler
+from sqlfluff.core.rules.doc_decorators import document_fix_compatible, document_groups
 from sqlfluff.rules.L023 import Rule_L023
 
 
+@document_groups
 @document_fix_compatible
 class Rule_L024(Rule_L023):
     """Single whitespace expected after ``USING`` in ``JOIN`` clause.
@@ -30,8 +32,7 @@ class Rule_L024(Rule_L023):
         LEFT JOIN zoo USING (a)
     """
 
-    expected_mother_segment_type = "join_clause"
-    pre_segment_identifier = ("name", "using")
-    post_segment_identifier = ("type", "bracketed")
-    expand_children = None
-    allow_newline = True
+    groups = ("all", "core")
+    crawl_behaviour = SegmentSeekerCrawler({"join_clause"})
+    target_keyword = "USING"
+    strip_newlines = False
