@@ -1,16 +1,19 @@
 """Implementation of Rule L030."""
 
 from typing import List, Tuple
+from sqlfluff.core.rules.crawlers import SegmentSeekerCrawler
 
 from sqlfluff.core.rules.doc_decorators import (
     document_configuration,
     document_fix_compatible,
+    document_groups,
 )
 from sqlfluff.rules.L010 import Rule_L010
 
 
-@document_configuration
+@document_groups
 @document_fix_compatible
+@document_configuration
 class Rule_L030(Rule_L010):
     """Inconsistent capitalisation of function names.
 
@@ -38,9 +41,16 @@ class Rule_L030(Rule_L010):
 
     """
 
-    _target_elems: List[Tuple[str, str]] = [
-        ("type", "function_name_identifier"),
-        ("type", "bare_function"),
+    groups = ("all", "core")
+    lint_phase = "post"
+    crawl_behaviour = SegmentSeekerCrawler(
+        {"function_name_identifier", "bare_function"}
+    )
+    _exclude_elements: List[Tuple[str, str]] = []
+    config_keywords = [
+        "extended_capitalisation_policy",
+        "ignore_words",
+        "ignore_words_regex",
     ]
     _description_elem = "Function names"
 
