@@ -1,9 +1,12 @@
 """Implementation of Rule L055."""
 from typing import Optional
 
-from sqlfluff.core.rules.base import BaseRule, LintResult, RuleContext
+from sqlfluff.core.rules import BaseRule, LintResult, RuleContext
+from sqlfluff.core.rules.crawlers import SegmentSeekerCrawler
+from sqlfluff.core.rules.doc_decorators import document_groups
 
 
+@document_groups
 class Rule_L055(BaseRule):
     """Use ``LEFT JOIN`` instead of ``RIGHT JOIN``.
 
@@ -36,15 +39,17 @@ class Rule_L055(BaseRule):
             ON foo.bar_id = bar.id;
     """
 
+    groups = ("all",)
+    crawl_behaviour = SegmentSeekerCrawler({"join_clause"})
+
     def _eval(self, context: RuleContext) -> Optional[LintResult]:
         """Use LEFT JOIN instead of RIGHT JOIN."""
         # We are only interested in JOIN clauses.
-        if context.segment.type != "join_clause":
-            return None
+        assert context.segment.is_type("join_clause")
 
         # Identify if RIGHT JOIN is present.
-        if {"right", "join"}.issubset(
-            {segment.name for segment in context.segment.segments}
+        if {"RIGHT", "JOIN"}.issubset(
+            {segment.raw_upper for segment in context.segment.segments}
         ):
             return LintResult(context.segment.segments[0])
 
